@@ -168,11 +168,13 @@ typedef struct VBOXHDDRAW
 /** Open image for asynchronous access.
  *  Only available if VD_CAP_ASYNC_IO is set
  *  Check with VDIsAsynchonousIoSupported wether
- *  asynchronous I/O is really supported for this file.
- */
+ *  asynchronous I/O is really supported for this file.  */
 #define VD_OPEN_FLAGS_ASYNC_IO      RT_BIT(4)
+/** Allow sharing of the image for writable images. May be ignored if the
+ * format backend doesn't support this type of concurrent access. */
+#define VD_OPEN_FLAGS_SHAREABLE     RT_BIT(5)
 /** Mask of valid flags. */
-#define VD_OPEN_FLAGS_MASK          (VD_OPEN_FLAGS_NORMAL | VD_OPEN_FLAGS_READONLY | VD_OPEN_FLAGS_HONOR_ZEROES | VD_OPEN_FLAGS_HONOR_SAME | VD_OPEN_FLAGS_INFO | VD_OPEN_FLAGS_ASYNC_IO)
+#define VD_OPEN_FLAGS_MASK          (VD_OPEN_FLAGS_NORMAL | VD_OPEN_FLAGS_READONLY | VD_OPEN_FLAGS_HONOR_ZEROES | VD_OPEN_FLAGS_HONOR_SAME | VD_OPEN_FLAGS_INFO | VD_OPEN_FLAGS_ASYNC_IO | VD_OPEN_FLAGS_SHAREABLE)
 /** @}*/
 
 
@@ -463,9 +465,11 @@ typedef DECLCALLBACK(int) FNVDCOMPLETED(void *pvUser, int rcReq);
 typedef FNVDCOMPLETED *PFNVDCOMPLETED;
 
 /** Open the storage readonly. */
-#define VD_INTERFACEASYNCIO_OPEN_FLAGS_READONLY RT_BIT(0)
+#define VD_INTERFACEASYNCIO_OPEN_FLAGS_READONLY  RT_BIT(0)
 /** Create the storage backend if it doesn't exist. */
-#define VD_INTERFACEASYNCIO_OPEN_FLAGS_CREATE RT_BIT(1)
+#define VD_INTERFACEASYNCIO_OPEN_FLAGS_CREATE    RT_BIT(1)
+/** Don't write lock the opened file. */
+#define VD_INTERFACEASYNCIO_OPEN_FLAGS_DONT_LOCK RT_BIT(2)
 
 /**
  * Support interface for asynchronous I/O
@@ -976,6 +980,8 @@ DECLINLINE(int) VDCFGQueryBytesAlloc(PVDINTERFACECONFIG pCfgIf,
     int rc = pCfgIf->pfnQuerySize(pvUser, pszName, &cb);
     if (RT_SUCCESS(rc))
     {
+        Assert(cb);
+
         char *pvData = (char *)RTMemAlloc(cb);
         if (pvData)
         {
@@ -983,7 +989,7 @@ DECLINLINE(int) VDCFGQueryBytesAlloc(PVDINTERFACECONFIG pCfgIf,
             if (RT_SUCCESS(rc))
             {
                 *ppvData = pvData;
-                *pcbData = cb;
+                *pcbData = cb - 1; /* Exclude terminator of the queried string. */
             }
             else
                 RTMemFree(pvData);
@@ -1057,16 +1063,24 @@ ion.
     DECLR3CALLBACKMEMBER(int, pfnRead, (RTSOCKET Sock, void *pvBuffer, size_t cbBuffer, size_t *pcbRead));
 
     /**
-     * Send data from a socket.
+     * Send data to a socket.
      *
      * @return  iprt status code.
      * @param   Sock        Socket descriptor.
      * @param   pvBuffer    Buffer to write data to socket.
      * @param   cbBuffer    How much to write.
-     * @param   pcbRead     Number of bytes read.
      */
     DECLR3CALLBACKMEMBER(int, pfnWrite, (RTSOCKET Sock, const void *pvBuffer, size_t cbBuffer));
 
+    /**
+     * Send data from scatter/gather buffer to a socket.
+     *
+     * @return  iprt status code.
+     * @param   Sock        Socket descriptor.
+     * @param   pSgBuf      Scatter/gather buffer to write data to socket.
+     */
+    DECLR3CALLBACKMEMBER(int, pfnSgWrite, (RTSOCKET Sock, PCRTSGBUF pSgBuf));
+
     /**
      * Flush socket write buffers.
      *
@@ -1075,6 +1089,15 @@ ion.
      */
     DECLR3CALLBACKMEMBER(int, pfnFlush, (RTSOCKET Sock));
 
+    /**
+     * Enables or disables delaying sends to coalesce packets.
+     *
+     * @return  iprt status code.
+     * @param   Sock        Socket descriptor.
+     * @param   fEnable     When set to true enables coalescing.
+     */
+    DECLR3CALLBACKMEMBER(int, pfnSetSendCoalescing, (RTSOCKET Sock, bool fEnable));
+
     /**
      * Gets the address of the local side.
      *
