@@ -3,7 +3,7 @@
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -21,10 +21,6 @@
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 #ifndef ___VBox_scsi_h
@@ -189,6 +185,7 @@ typedef enum SCSISVCACTIONIN
 #define SCSI_ASC_MEDIUM_MAY_HAVE_CHANGED            0x28
 #define SCSI_ASC_MEDIUM_NOT_PRESENT                 0x3a
 #define SCSI_ASC_SAVING_PARAMETERS_NOT_SUPPORTED    0x39
+#define SCSI_ASC_INVALID_MESSAGE                    0x49
 #define SCSI_ASC_MEDIA_LOAD_OR_EJECT_FAILED         0x53
 #define SCSI_ASC_LOGICAL_UNIT_DOES_NOT_RESPOND_TO_SELECTION 0x00
 
