@@ -437,6 +437,13 @@ void VBoxServiceMainWait(void)
 {
     int rc;
 
+    /* Report the host that we're up and running! */
+    rc = VbglR3ReportAdditionsStatus(VBoxGuestStatusFacility_VBoxService,
+                                     VBoxGuestStatusCurrent_Active,
+                                     0); /* Flags; not used. */
+    if (RT_FAILURE(rc))
+        VBoxServiceError("Failed to report Guest Additions status to the host! rc=%Rrc\n", rc);
+
 #ifdef RT_OS_WINDOWS
     /*
      * Wait for the semaphore to be signalled.
@@ -505,6 +512,16 @@ int main(int argc, char **argv)
     if (RT_FAILURE(rc))
         return VBoxServiceError("VbglR3Init failed with rc=%Rrc.\n", rc);
 
+#ifdef RT_OS_WINDOWS
+    /* Special forked VBoxService.exe process for handling page fusion. */
+    /* Note: ugly hack, but annoying to install additional executables. */
+    if (    argc == 2
+        &&  !strcmp(argv[1], "-pagefusionfork"))
+    {
+        return VBoxServicePageSharingInitFork();
+    }
+#endif
+
     /* Do pre-init of services. */
     g_pszProgName = RTPathFilename(argv[0]);
     for (unsigned j = 0; j < RT_ELEMENTS(g_aServices); j++)
