@@ -483,9 +483,11 @@ static int VBoxServiceControlExecProcLoop(PVBOXSERVICECTRLTHREAD pThread,
         rc = VbglR3GuestCtrlExecReportStatus(pThread->uClientID, pThread->uContextID,
                                              pData->uPID, uStatus, uFlags,
                                              NULL /* pvData */, 0 /* cbData */);
+        VBoxServiceVerbose(3, "ControlExec: Process loop ended with rc=%Rrc\n", rc);
     }
+    else
+        VBoxServiceError("ControlExec: Process loop failed with rc=%Rrc\n", rc);
     RTMemFree(StdInBuf.pch);
-    VBoxServiceVerbose(3, "ControlExec: Process loop ended with rc=%Rrc\n", rc);
     return rc;
 }
 
@@ -795,20 +797,18 @@ DECLCALLBACK(int) VBoxServiceControlExecProcessWorker(PVBOXSERVICECTRLTHREAD pTh
     PVBOXSERVICECTRLTHREADDATAEXEC pData = (PVBOXSERVICECTRLTHREADDATAEXEC)pThread->pvData;
     AssertPtr(pData);
 
-    /*
-     * Tell the control thread that it can continue
-     * spawning services.
-     */
-    RTThreadUserSignal(RTThreadSelf());
     VBoxServiceVerbose(3, "ControlExec: Thread of process \"%s\" started\n", pData->pszCmd);
 
     int rc = VbglR3GuestCtrlConnect(&pThread->uClientID);
     if (RT_FAILURE(rc))
     {
         VBoxServiceError("ControlExec: Thread failed to connect to the guest control service, aborted! Error: %Rrc\n", rc);
+        RTThreadUserSignal(RTThreadSelf());
         return rc;
     }
 
+    bool fSignalled = false; /* Indicator whether we signalled the thread user event already. */
+
     /*
      * Create the environment.
      */
@@ -867,12 +867,24 @@ DECLCALLBACK(int) VBoxServiceControlExecProcessWorker(PVBOXSERVICECTRLTHREAD pTh
                                                                          phStdIn, phStdOut, phStdErr,
                                                                          pData->pszUser, pData->pszPassword,
                                                                          &hProcess);
+
+                                /*
+                                 * Tell the control thread that it can continue
+                                 * spawning services. This needs to be done after the new
+                                 * process has been started because otherwise signal handling
+                                 * on (Open) Solaris does not work correctly (see #5068).
+                                 */
+                                int rc2 = RTThreadUserSignal(RTThreadSelf());
+                                if (RT_FAILURE(rc2))
+                                    rc = rc2;
+                                fSignalled = true;
+
                                 if (RT_SUCCESS(rc))
                                 {
                                     /*
                                      * Close the child ends of any pipes and redirected files.
                                      */
-                                    int rc2 = RTHandleClose(phStdIn);   AssertRC(rc2);
+                                    rc2 = RTHandleClose(phStdIn);   AssertRC(rc2);
                                     phStdIn    = NULL;
                                     rc2 = RTHandleClose(phStdOut);  AssertRC(rc2);
                                     phStdOut   = NULL;
@@ -902,9 +914,9 @@ DECLCALLBACK(int) VBoxServiceControlExecProcessWorker(PVBOXSERVICECTRLTHREAD pTh
                                     VBoxServiceError("ControlExec: Could not start process '%s' (CID: %u)! Error: %Rrc\n",
                                                      pData->pszCmd, pThread->uContextID, rc);
 
-                                    int rc2 = VbglR3GuestCtrlExecReportStatus(pThread->uClientID, pThread->uContextID, pData->uPID,
-                                                                              PROC_STS_ERROR, rc,
-                                                                              NULL /* pvData */, 0 /* cbData */);
+                                    rc2 = VbglR3GuestCtrlExecReportStatus(pThread->uClientID, pThread->uContextID, pData->uPID,
+                                                                          PROC_STS_ERROR, rc,
+                                                                          NULL /* pvData */, 0 /* cbData */);
                                     if (RT_FAILURE(rc2))
                                         VBoxServiceError("ControlExec: Could not report process start error! Error: %Rrc (process error %Rrc)\n",
                                                          rc2, rc);
@@ -927,6 +939,13 @@ DECLCALLBACK(int) VBoxServiceControlExecProcessWorker(PVBOXSERVICECTRLTHREAD pTh
     VbglR3GuestCtrlDisconnect(pThread->uClientID);
     VBoxServiceVerbose(3, "ControlExec: Thread of process \"%s\" (PID: %u) ended with rc=%Rrc\n",
                        pData->pszCmd, pData->uPID, rc);
+
+    /*
+     * If something went wrong signal the user event so that others don't wait
+     * forever on this thread.
+     */
+    if (RT_FAILURE(rc) && !fSignalled)
+        RTThreadUserSignal(RTThreadSelf());
     return rc;
 }
 
@@ -966,6 +985,8 @@ int VBoxServiceControlExecProcess(uint32_t uContextID, const char *pszCmd, uint3
             }
             else
             {
+                VBoxServiceVerbose(4, "ControlExec: Waiting for thread to initialize ...\n");
+
                 /* Wait for the thread to initialize. */
                 RTThreadUserWait(pThread->Thread, 60 * 1000);
                 if (pThread->fShutdown)
