@@ -34,74 +34,74 @@
  *   http://pserver.samba.org/samba/ftp/cifs-cvs/samplefs.tar.gz
  */
 
-#if LINUX_VERSION_CODE < KERNEL_VERSION (2, 6, 0)
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 0)
 static void
-sf_ftime_from_timespec (time_t *time, RTTIMESPEC *ts)
+sf_ftime_from_timespec(time_t *time, RTTIMESPEC *ts)
 {
-        int64_t t = RTTimeSpecGetNano (ts);
+        int64_t t = RTTimeSpecGetNano(ts);
 
-        do_div (t, 1000000000);
+        do_div(t, 1000000000);
         *time = t;
 }
 
 static void
-sf_timespec_from_ftime (RTTIMESPEC *ts, time_t *time)
+sf_timespec_from_ftime(RTTIMESPEC *ts, time_t *time)
 {
         int64_t t = 1000000000 * *time;
-        RTTimeSpecSetNano (ts, t);
+        RTTimeSpecSetNano(ts, t);
 }
 #else /* >= 2.6.0 */
 static void
-sf_ftime_from_timespec (struct timespec *tv, RTTIMESPEC *ts)
+sf_ftime_from_timespec(struct timespec *tv, RTTIMESPEC *ts)
 {
-        int64_t t = RTTimeSpecGetNano (ts);
+        int64_t t = RTTimeSpecGetNano(ts);
         int64_t nsec;
 
-        nsec = do_div (t, 1000000000);
+        nsec = do_div(t, 1000000000);
         tv->tv_sec = t;
         tv->tv_nsec = nsec;
 }
 
 static void
-sf_timespec_from_ftime (RTTIMESPEC *ts, struct timespec *tv)
+sf_timespec_from_ftime(RTTIMESPEC *ts, struct timespec *tv)
 {
         int64_t t = (int64_t)tv->tv_nsec + (int64_t)tv->tv_sec * 1000000000;
-        RTTimeSpecSetNano (ts, t);
+        RTTimeSpecSetNano(ts, t);
 }
 #endif /* >= 2.6.0 */
 
 /* set [inode] attributes based on [info], uid/gid based on [sf_g] */
 void
-sf_init_inode (struct sf_glob_info *sf_g, struct inode *inode,
+sf_init_inode(struct sf_glob_info *sf_g, struct inode *inode,
                RTFSOBJINFO *info)
 {
         int is_dir;
         RTFSOBJATTR *attr;
         int mode;
 
-        TRACE ();
+        TRACE();
 
         attr = &info->Attr;
-        is_dir = RTFS_IS_DIRECTORY (attr->fMode);
+        is_dir = RTFS_IS_DIRECTORY(attr->fMode);
 
 #define mode_set(r) attr->fMode & (RTFS_UNIX_##r) ? (S_##r) : 0;
-        mode = mode_set (ISUID);
-        mode |= mode_set (ISGID);
+        mode = mode_set(ISUID);
+        mode |= mode_set(ISGID);
 
-        mode |= mode_set (IRUSR);
-        mode |= mode_set (IWUSR);
-        mode |= mode_set (IXUSR);
+        mode |= mode_set(IRUSR);
+        mode |= mode_set(IWUSR);
+        mode |= mode_set(IXUSR);
 
-        mode |= mode_set (IRGRP);
-        mode |= mode_set (IWGRP);
-        mode |= mode_set (IXGRP);
+        mode |= mode_set(IRGRP);
+        mode |= mode_set(IWGRP);
+        mode |= mode_set(IXGRP);
 
-        mode |= mode_set (IROTH);
-        mode |= mode_set (IWOTH);
-        mode |= mode_set (IXOTH);
+        mode |= mode_set(IROTH);
+        mode |= mode_set(IWOTH);
+        mode |= mode_set(IXOTH);
 #undef mode_set
 
-#if LINUX_VERSION_CODE >= KERNEL_VERSION (2, 6, 0)
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 0)
         inode->i_mapping->a_ops = &sf_reg_aops;
         inode->i_mapping->backing_dev_info = &sf_g->bdi;
 #endif
@@ -128,35 +128,35 @@ sf_init_inode (struct sf_glob_info *sf_g, struct inode *inode,
         inode->i_uid = sf_g->uid;
         inode->i_gid = sf_g->gid;
         inode->i_size = info->cbObject;
-#if LINUX_VERSION_CODE < KERNEL_VERSION (2, 6, 19) && !defined(KERNEL_FC6)
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 19) && !defined(KERNEL_FC6)
         inode->i_blksize = 4096;
 #endif
-#if LINUX_VERSION_CODE >= KERNEL_VERSION (2, 4, 11)
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 4, 11)
         inode->i_blkbits = 12;
 #endif
         /* i_blocks always in units of 512 bytes! */
         inode->i_blocks = (info->cbAllocated + 511) / 512;
 
-        sf_ftime_from_timespec (&inode->i_atime, &info->AccessTime);
-        sf_ftime_from_timespec (&inode->i_ctime, &info->ChangeTime);
-        sf_ftime_from_timespec (&inode->i_mtime, &info->ModificationTime);
+        sf_ftime_from_timespec(&inode->i_atime, &info->AccessTime);
+        sf_ftime_from_timespec(&inode->i_ctime, &info->ChangeTime);
+        sf_ftime_from_timespec(&inode->i_mtime, &info->ModificationTime);
 }
 
 int
-sf_stat (const char *caller, struct sf_glob_info *sf_g,
-         SHFLSTRING *path, RTFSOBJINFO *result, int ok_to_fail)
+sf_stat(const char *caller, struct sf_glob_info *sf_g,
+        SHFLSTRING *path, RTFSOBJINFO *result, int ok_to_fail)
 {
         int rc;
         SHFLCREATEPARMS params;
 
-        TRACE ();
+        TRACE();
 
         RT_ZERO(params);
         params.CreateFlags = SHFL_CF_LOOKUP | SHFL_CF_ACT_FAIL_IF_NEW;
         LogFunc(("sf_stat: calling vboxCallCreate, file %s, flags %#x\n",
                  path->String.utf8, params.CreateFlags));
-        rc = vboxCallCreate (&client_handle, &sf_g->map, path, &params);
-        if (RT_FAILURE (rc)) {
+        rc = vboxCallCreate(&client_handle, &sf_g->map, path, &params);
+        if (RT_FAILURE(rc)) {
                 LogFunc(("vboxCallCreate(%s) failed.  caller=%s, rc=%Rrc\n",
                          path->String.utf8, rc, caller));
                 return -EPROTO;
@@ -181,31 +181,31 @@ sf_stat (const char *caller, struct sf_glob_info *sf_g,
    or [sf_stat] is unsuccessful, otherwise we return success and
    update inode attributes */
 int
-sf_inode_revalidate (struct dentry *dentry)
+sf_inode_revalidate(struct dentry *dentry)
 {
         int err;
         struct sf_glob_info *sf_g;
         struct sf_inode_info *sf_i;
         RTFSOBJINFO info;
 
-        TRACE ();
+        TRACE();
         if (!dentry || !dentry->d_inode) {
                 LogFunc(("no dentry(%p) or inode(%p)\n", dentry, dentry->d_inode));
                 return -EINVAL;
         }
 
-        sf_g = GET_GLOB_INFO (dentry->d_inode->i_sb);
-        sf_i = GET_INODE_INFO (dentry->d_inode);
+        sf_g = GET_GLOB_INFO(dentry->d_inode->i_sb);
+        sf_i = GET_INODE_INFO(dentry->d_inode);
 
 #if 0
-        printk ("%s called by %p:%p\n",
-                sf_i->path->String.utf8,
-                __builtin_return_address (0),
-                __builtin_return_address (1));
+        printk("%s called by %p:%p\n",
+               sf_i->path->String.utf8,
+               __builtin_return_address (0),
+               __builtin_return_address (1));
 #endif
 
-        BUG_ON (!sf_g);
-        BUG_ON (!sf_i);
+        BUG_ON(!sf_g);
+        BUG_ON(!sf_i);
 
         if (!sf_i->force_restat) {
                 if (jiffies - dentry->d_time < sf_g->ttl) {
@@ -213,13 +213,13 @@ sf_inode_revalidate (struct dentry *dentry)
                 }
         }
 
-        err = sf_stat (__func__, sf_g, sf_i->path, &info, 1);
+        err = sf_stat(__func__, sf_g, sf_i->path, &info, 1);
         if (err) {
                 return err;
         }
 
         dentry->d_time = jiffies;
-        sf_init_inode (sf_g, dentry->d_inode, &info);
+        sf_init_inode(sf_g, dentry->d_inode, &info);
         return 0;
 }
 
@@ -227,14 +227,14 @@ sf_inode_revalidate (struct dentry *dentry)
    [dentry] in the cache is still valid. the job is handled by
    [sf_inode_revalidate] */
 static int
-#if LINUX_VERSION_CODE < KERNEL_VERSION (2, 6, 0)
-sf_dentry_revalidate (struct dentry *dentry, int flags)
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 0)
+sf_dentry_revalidate(struct dentry *dentry, int flags)
 #else
-sf_dentry_revalidate (struct dentry *dentry, struct nameidata *nd)
+sf_dentry_revalidate(struct dentry *dentry, struct nameidata *nd)
 #endif
 {
-        TRACE ();
-        if (sf_inode_revalidate (dentry)) {
+        TRACE();
+        if (sf_inode_revalidate(dentry)) {
                 return 0;
         }
         return 1;
@@ -244,24 +244,24 @@ sf_dentry_revalidate (struct dentry *dentry, struct nameidata *nd)
    effect) updates inode attributes for [dentry] (given that [dentry]
    has inode at all) from these new attributes we derive [kstat] via
    [generic_fillattr] */
-#if LINUX_VERSION_CODE >= KERNEL_VERSION (2, 6, 0)
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 0)
 int
-sf_getattr (struct vfsmount *mnt, struct dentry *dentry, struct kstat *kstat)
+sf_getattr(struct vfsmount *mnt, struct dentry *dentry, struct kstat *kstat)
 {
         int err;
 
-        TRACE ();
-        err = sf_inode_revalidate (dentry);
+        TRACE();
+        err = sf_inode_revalidate(dentry);
         if (err) {
                 return err;
         }
 
-        generic_fillattr (dentry->d_inode, kstat);
+        generic_fillattr(dentry->d_inode, kstat);
         return 0;
 }
 
 int
-sf_setattr (struct dentry *dentry, struct iattr *iattr)
+sf_setattr(struct dentry *dentry, struct iattr *iattr)
 {
         struct sf_glob_info *sf_g;
         struct sf_inode_info *sf_i;
@@ -270,10 +270,10 @@ sf_setattr (struct dentry *dentry, struct iattr *iattr)
         uint32_t cbBuffer;
         int rc, err;
 
-        TRACE ();
+        TRACE();
 
-        sf_g = GET_GLOB_INFO (dentry->d_inode->i_sb);
-        sf_i = GET_INODE_INFO (dentry->d_inode);
+        sf_g = GET_GLOB_INFO(dentry->d_inode->i_sb);
+        sf_i = GET_INODE_INFO(dentry->d_inode);
         err  = 0;
 
         RT_ZERO(params);
@@ -286,8 +286,8 @@ sf_setattr (struct dentry *dentry, struct iattr *iattr)
         if (iattr->ia_valid & ATTR_SIZE)
             params.CreateFlags |= SHFL_CF_ACCESS_WRITE;
 
-        rc = vboxCallCreate (&client_handle, &sf_g->map, sf_i->path, &params);
-        if (RT_FAILURE (rc)) {
+        rc = vboxCallCreate(&client_handle, &sf_g->map, sf_i->path, &params);
+        if (RT_FAILURE(rc)) {
                 LogFunc(("vboxCallCreate(%s) failed rc=%Rrc\n",
                         sf_i->path->String.utf8, rc));
                 err = -RTErrConvertToErrno(rc);
@@ -309,17 +309,17 @@ sf_setattr (struct dentry *dentry, struct iattr *iattr)
             RT_ZERO(info);
             if (iattr->ia_valid & ATTR_MODE)
             {
-                info.Attr.fMode  = mode_set (ISUID);
-                info.Attr.fMode |= mode_set (ISGID);
-                info.Attr.fMode |= mode_set (IRUSR);
-                info.Attr.fMode |= mode_set (IWUSR);
-                info.Attr.fMode |= mode_set (IXUSR);
-                info.Attr.fMode |= mode_set (IRGRP);
-                info.Attr.fMode |= mode_set (IWGRP);
-                info.Attr.fMode |= mode_set (IXGRP);
-                info.Attr.fMode |= mode_set (IROTH);
-                info.Attr.fMode |= mode_set (IWOTH);
-                info.Attr.fMode |= mode_set (IXOTH);
+                info.Attr.fMode  = mode_set(ISUID);
+                info.Attr.fMode |= mode_set(ISGID);
+                info.Attr.fMode |= mode_set(IRUSR);
+                info.Attr.fMode |= mode_set(IWUSR);
+                info.Attr.fMode |= mode_set(IXUSR);
+                info.Attr.fMode |= mode_set(IRGRP);
+                info.Attr.fMode |= mode_set(IWGRP);
+                info.Attr.fMode |= mode_set(IXGRP);
+                info.Attr.fMode |= mode_set(IROTH);
+                info.Attr.fMode |= mode_set(IWOTH);
+                info.Attr.fMode |= mode_set(IXOTH);
 
                 if (iattr->ia_mode & S_IFDIR)
                     info.Attr.fMode |= RTFS_TYPE_DIRECTORY;
@@ -328,16 +328,16 @@ sf_setattr (struct dentry *dentry, struct iattr *iattr)
             }
 
             if (iattr->ia_valid & ATTR_ATIME)
-                sf_timespec_from_ftime (&info.AccessTime, &iattr->ia_atime);
+                sf_timespec_from_ftime(&info.AccessTime, &iattr->ia_atime);
             if (iattr->ia_valid & ATTR_MTIME)
-                sf_timespec_from_ftime (&info.ModificationTime, &iattr->ia_mtime);
+                sf_timespec_from_ftime(&info.ModificationTime, &iattr->ia_mtime);
             /* ignore ctime (inode change time) as it can't be set from userland anyway */
 
             cbBuffer = sizeof(info);
             rc = vboxCallFSInfo(&client_handle, &sf_g->map, params.Handle,
                                 SHFL_INFO_SET | SHFL_INFO_FILE, &cbBuffer,
                                 (PSHFLDIRINFO)&info);
-            if (RT_FAILURE (rc)) {
+            if (RT_FAILURE(rc)) {
                 LogFunc(("vboxCallFSInfo(%s, FILE) failed rc=%Rrc\n",
                         sf_i->path->String.utf8, rc));
                 err = -RTErrConvertToErrno(rc);
@@ -353,7 +353,7 @@ sf_setattr (struct dentry *dentry, struct iattr *iattr)
             rc = vboxCallFSInfo(&client_handle, &sf_g->map, params.Handle,
                                 SHFL_INFO_SET | SHFL_INFO_SIZE, &cbBuffer,
                                 (PSHFLDIRINFO)&info);
-            if (RT_FAILURE (rc)) {
+            if (RT_FAILURE(rc)) {
                 LogFunc(("vboxCallFSInfo(%s, SIZE) failed rc=%Rrc\n",
                         sf_i->path->String.utf8, rc));
                 err = -RTErrConvertToErrno(rc);
@@ -361,18 +361,18 @@ sf_setattr (struct dentry *dentry, struct iattr *iattr)
             }
         }
 
-        rc = vboxCallClose (&client_handle, &sf_g->map, params.Handle);
-        if (RT_FAILURE (rc))
+        rc = vboxCallClose(&client_handle, &sf_g->map, params.Handle);
+        if (RT_FAILURE(rc))
         {
                 LogFunc(("vboxCallClose(%s) failed rc=%Rrc\n",
                       sf_i->path->String.utf8, rc));
         }
 
-        return sf_inode_revalidate (dentry);
+        return sf_inode_revalidate(dentry);
 
 fail1:
-        rc = vboxCallClose (&client_handle, &sf_g->map, params.Handle);
-        if (RT_FAILURE (rc))
+        rc = vboxCallClose(&client_handle, &sf_g->map, params.Handle);
+        if (RT_FAILURE(rc))
         {
                 LogFunc(("vboxCallClose(%s) failed rc=%Rrc\n",
                       sf_i->path->String.utf8, rc));
@@ -383,7 +383,7 @@ fail2:
 #endif /* >= 2.6.0 */
 
 static int
-sf_make_path (const char *caller, struct sf_inode_info *sf_i,
+sf_make_path(const char *caller, struct sf_inode_info *sf_i,
               const char *d_name, size_t d_len, SHFLSTRING **result)
 {
         size_t path_len, shflstring_len;
@@ -393,7 +393,7 @@ sf_make_path (const char *caller, struct sf_inode_info *sf_i,
         uint8_t *dst;
         int is_root = 0;
 
-        TRACE ();
+        TRACE();
         p_len = sf_i->path->u16Length;
         p_name = sf_i->path->String.utf8;
 
@@ -410,8 +410,8 @@ sf_make_path (const char *caller, struct sf_inode_info *sf_i,
                 }
         }
 
-        shflstring_len = offsetof (SHFLSTRING, String.utf8) + path_len;
-        tmp = kmalloc (shflstring_len, GFP_KERNEL);
+        shflstring_len = offsetof(SHFLSTRING, String.utf8) + path_len;
+        tmp = kmalloc(shflstring_len, GFP_KERNEL);
         if (!tmp) {
                 LogRelFunc(("kmalloc failed, caller=%s\n", caller));
                 return -ENOMEM;
@@ -420,13 +420,13 @@ sf_make_path (const char *caller, struct sf_inode_info *sf_i,
         tmp->u16Size = path_len;
 
         if (is_root) {
-                memcpy (tmp->String.utf8, d_name, d_len + 1);
+                memcpy(tmp->String.utf8, d_name, d_len + 1);
         }
         else {
                 dst = tmp->String.utf8;
-                memcpy (dst, p_name, p_len);
+                memcpy(dst, p_name, p_len);
                 dst += p_len; *dst++ = '/';
-                memcpy (dst, d_name, d_len);
+                memcpy(dst, d_name, d_len);
                 dst[d_len] = 0;
         }
 
@@ -438,7 +438,7 @@ sf_make_path (const char *caller, struct sf_inode_info *sf_i,
    to [sf_g]->nls, we must convert it to UTF8 here and pass down to
    [sf_make_path] which will allocate SHFLSTRING and fill it in */
 int
-sf_path_from_dentry (const char *caller, struct sf_glob_info *sf_g,
+sf_path_from_dentry(const char *caller, struct sf_glob_info *sf_g,
                      struct sf_inode_info *sf_i, struct dentry *dentry,
                      SHFLSTRING **result)
 {
@@ -448,7 +448,7 @@ sf_path_from_dentry (const char *caller, struct sf_glob_info *sf_g,
         const char *name;
         size_t len = 0;
 
-        TRACE ();
+        TRACE();
         d_name = dentry->d_name.name;
         d_len = dentry->d_name.len;
 
@@ -461,7 +461,7 @@ sf_path_from_dentry (const char *caller, struct sf_glob_info *sf_g,
                 in_len = d_len;
 
                 out_bound_len = PATH_MAX;
-                out = kmalloc (out_bound_len, GFP_KERNEL);
+                out = kmalloc(out_bound_len, GFP_KERNEL);
                 name = out;
 
                 for (i = 0; i < d_len; ++i) {
@@ -470,7 +470,7 @@ sf_path_from_dentry (const char *caller, struct sf_glob_info *sf_g,
                         linux_wchar_t uni;
                         int nb;
 
-                        nb = sf_g->nls->char2uni (in, in_len, &uni);
+                        nb = sf_g->nls->char2uni(in, in_len, &uni);
                         if (nb < 0) {
                                 LogFunc(("nls->char2uni failed %x %d\n",
                                          *in, in_len));
@@ -480,10 +480,10 @@ sf_path_from_dentry (const char *caller, struct sf_glob_info *sf_g,
                         in_len -= nb;
                         in += nb;
 
-#if LINUX_VERSION_CODE >= KERNEL_VERSION (2, 6, 31)
-                        nb = utf32_to_utf8 (uni, out, out_bound_len);
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 31)
+                        nb = utf32_to_utf8(uni, out, out_bound_len);
 #else
-                        nb = utf8_wctomb (out, uni, out_bound_len);
+                        nb = utf8_wctomb(out, uni, out_bound_len);
 #endif
                         if (nb < 0) {
                                 LogFunc(("nls->uni2char failed %x %d\n",
@@ -508,19 +508,19 @@ sf_path_from_dentry (const char *caller, struct sf_glob_info *sf_g,
                 len = d_len;
         }
 
-        err = sf_make_path (caller, sf_i, name, len, result);
+        err = sf_make_path(caller, sf_i, name, len, result);
         if (name != d_name) {
-                kfree (name);
+                kfree(name);
         }
         return err;
 
  fail1:
-        kfree (name);
+        kfree(name);
         return err;
 }
 
 int
-sf_nlscpy (struct sf_glob_info *sf_g,
+sf_nlscpy(struct sf_glob_info *sf_g,
            char *name, size_t name_bound_len,
            const unsigned char *utf8_name, size_t utf8_len)
 {
@@ -542,10 +542,10 @@ sf_nlscpy (struct sf_glob_info *sf_g,
                         int nb;
                         wchar_t uni;
 
-#if LINUX_VERSION_CODE >= KERNEL_VERSION (2, 6, 31)
-                        nb = utf8_to_utf32 (in, in_bound_len, &uni);
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 31)
+                        nb = utf8_to_utf32(in, in_bound_len, &uni);
 #else
-                        nb = utf8_mbtowc (&uni, in, in_bound_len);
+                        nb = utf8_mbtowc(&uni, in, in_bound_len);
 #endif
                         if (nb < 0) {
                                 LogFunc(("utf8_mbtowc failed(%s) %x:%d\n",
@@ -555,7 +555,7 @@ sf_nlscpy (struct sf_glob_info *sf_g,
                         in += nb;
                         in_bound_len -= nb;
 
-                        nb = sf_g->nls->uni2char (uni, out, out_bound_len);
+                        nb = sf_g->nls->uni2char(uni, out, out_bound_len);
                         if (nb < 0) {
                                 LogFunc(("nls->uni2char failed(%s) %x:%d\n",
                                          utf8_name, uni, out_bound_len));
@@ -574,36 +574,36 @@ sf_nlscpy (struct sf_glob_info *sf_g,
                         return -ENAMETOOLONG;
                 }
                 else {
-                        memcpy (name, utf8_name, utf8_len + 1);
+                        memcpy(name, utf8_name, utf8_len + 1);
                 }
                 return 0;
         }
 }
 
 static struct sf_dir_buf *
-sf_dir_buf_alloc (void)
+sf_dir_buf_alloc(void)
 {
         struct sf_dir_buf *b;
 
-        TRACE ();
-        b = kmalloc (sizeof (*b), GFP_KERNEL);
+        TRACE();
+        b = kmalloc(sizeof(*b), GFP_KERNEL);
         if (!b) {
                 LogRelFunc(("could not alloc directory buffer\n"));
                 return NULL;
         }
 
 #ifdef USE_VMALLOC
-        b->buf = vmalloc (16384);
+        b->buf = vmalloc(16384);
 #else
-        b->buf = kmalloc (16384, GFP_KERNEL);
+        b->buf = kmalloc(16384, GFP_KERNEL);
 #endif
         if (!b->buf) {
-                kfree (b);
+                kfree(b);
                 LogRelFunc(("could not alloc directory buffer storage\n"));
                 return NULL;
         }
 
-        INIT_LIST_HEAD (&b->head);
+        INIT_LIST_HEAD(&b->head);
         b->nb_entries = 0;
         b->used_bytes = 0;
         b->free_bytes = 16384;
@@ -611,67 +611,82 @@ sf_dir_buf_alloc (void)
 }
 
 static void
-sf_dir_buf_free (struct sf_dir_buf *b)
+sf_dir_buf_free(struct sf_dir_buf *b)
 {
-        BUG_ON (!b || !b->buf);
+        BUG_ON(!b || !b->buf);
 
-        TRACE ();
-        list_del (&b->head);
+        TRACE();
+        list_del(&b->head);
 #ifdef USE_VMALLOC
-        vfree (b->buf);
+        vfree(b->buf);
 #else
-        kfree (b->buf);
+        kfree(b->buf);
 #endif
-        kfree (b);
+        kfree(b);
 }
 
 void
-sf_dir_info_free (struct sf_dir_info *p)
+sf_dir_info_free(struct sf_dir_info *p)
 {
         struct list_head *list, *pos, *tmp;
 
-        TRACE ();
+        TRACE();
         list = &p->info_list;
-        list_for_each_safe (pos, tmp, list) {
+        list_for_each_safe(pos, tmp, list) {
                 struct sf_dir_buf *b;
 
-                b = list_entry (pos, struct sf_dir_buf, head);
-                sf_dir_buf_free (b);
+                b = list_entry(pos, struct sf_dir_buf, head);
+                sf_dir_buf_free(b);
+        }
+        kfree(p);
+}
+
+void
+sf_dir_info_empty(struct sf_dir_info *p)
+{
+        struct list_head *list, *pos, *tmp;
+        TRACE();
+        list = &p->info_list;
+        list_for_each_safe(pos, tmp, list) {
+                struct sf_dir_buf *b;
+                b = list_entry(pos, struct sf_dir_buf, head);
+                b->nb_entries = 0;
+                b->used_bytes = 0;
+                b->free_bytes = 16384;
         }
-        kfree (p);
 }
 
 struct sf_dir_info *
-sf_dir_info_alloc (void)
+sf_dir_info_alloc(void)
 {
         struct sf_dir_info *p;
 
-        TRACE ();
-        p = kmalloc (sizeof (*p), GFP_KERNEL);
+        TRACE();
+        p = kmalloc(sizeof(*p), GFP_KERNEL);
         if (!p) {
                 LogRelFunc(("could not alloc directory info\n"));
                 return NULL;
         }
 
-        INIT_LIST_HEAD (&p->info_list);
+        INIT_LIST_HEAD(&p->info_list);
         return p;
 }
 
 static struct sf_dir_buf *
-sf_get_non_empty_dir_buf (struct sf_dir_info *sf_d)
+sf_get_empty_dir_buf(struct sf_dir_info *sf_d)
 {
         struct list_head *list, *pos;
 
         list = &sf_d->info_list;
-        list_for_each (pos, list) {
+        list_for_each(pos, list) {
                 struct sf_dir_buf *b;
 
-                b = list_entry (pos, struct sf_dir_buf, head);
+                b = list_entry(pos, struct sf_dir_buf, head);
                 if (!b) {
                         return NULL;
                 }
                 else {
-                        if (b->free_bytes > 0) {
+                        if (b->used_bytes == 0) {
                                 return b;
                         }
                 }
@@ -681,51 +696,41 @@ sf_get_non_empty_dir_buf (struct sf_dir_info *sf_d)
 }
 
 int
-sf_dir_read_all (struct sf_glob_info *sf_g, struct sf_inode_info *sf_i,
+sf_dir_read_all(struct sf_glob_info *sf_g, struct sf_inode_info *sf_i,
                  struct sf_dir_info *sf_d, SHFLHANDLE handle)
 {
         int err;
         SHFLSTRING *mask;
         struct sf_dir_buf *b;
 
-        TRACE ();
-        err = sf_make_path (__func__, sf_i, "*", 1, &mask);
+        TRACE();
+        err = sf_make_path(__func__, sf_i, "*", 1, &mask);
         if (err) {
                 goto fail0;
         }
 
-        b = sf_get_non_empty_dir_buf (sf_d);
         for (;;) {
                 int rc;
                 void *buf;
                 uint32_t buf_size;
                 uint32_t nb_ents;
 
+                b = sf_get_empty_dir_buf(sf_d);
                 if (!b) {
-                        b = sf_dir_buf_alloc ();
+                        b = sf_dir_buf_alloc();
                         if (!b) {
                                 err = -ENOMEM;
                                 LogRelFunc(("could not alloc directory buffer\n"));
                                 goto fail1;
                         }
+                        list_add(&b->head, &sf_d->info_list);
                 }
 
-                list_add (&b->head, &sf_d->info_list);
-
                 buf = b->buf;
                 buf_size = b->free_bytes;
 
-                rc = vboxCallDirInfo (
-                        &client_handle,
-                        &sf_g->map,
-                        handle,
-                        mask,
-                        0,
-                        0,
-                        &buf_size,
-                        buf,
-                        &nb_ents
-                        );
+                rc = vboxCallDirInfo(&client_handle, &sf_g->map, handle, mask,
+                                     0, 0, &buf_size, buf, &nb_ents);
                 switch (rc) {
                         case VINF_SUCCESS:
                                 /* fallthrough */
@@ -738,7 +743,7 @@ sf_dir_read_all (struct sf_glob_info *sf_g, struct sf_inode_info *sf_i,
                                 break;
 
                         default:
-                                err = -RTErrConvertToErrno (rc);
+                                err = -RTErrConvertToErrno(rc);
                                 LogFunc(("vboxCallDirInfo failed rc=%Rrc\n", rc));
                                 goto fail1;
                 }
@@ -746,16 +751,15 @@ sf_dir_read_all (struct sf_glob_info *sf_g, struct sf_inode_info *sf_i,
                 b->nb_entries += nb_ents;
                 b->free_bytes -= buf_size;
                 b->used_bytes += buf_size;
-                b = NULL;
 
-                if (RT_FAILURE (rc)) {
+                if (RT_FAILURE(rc))
                         break;
-                }
         }
-        return 0;
+        err = 0;
 
  fail1:
-        kfree (mask);
+        kfree(mask);
+
  fail0:
         return err;
 }
@@ -767,7 +771,7 @@ int sf_get_volume_info(struct super_block *sb, STRUCT_STATFS *stat)
         uint32_t cbBuffer;
         int rc;
 
-        sf_g = GET_GLOB_INFO (sb);
+        sf_g = GET_GLOB_INFO(sb);
         cbBuffer = sizeof(SHFLVolumeInfo);
         rc = vboxCallFSInfo(&client_handle, &sf_g->map, 0, SHFL_INFO_GET | SHFL_INFO_VOLUME,
                             &cbBuffer, (PSHFLDIRINFO)&SHFLVolumeInfo);
@@ -799,18 +803,18 @@ int sf_init_backing_dev(struct sf_glob_info *sf_g, const char *name)
 {
     int rc = 0;
 
-#if LINUX_VERSION_CODE >= KERNEL_VERSION (2, 6, 0)
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 0)
     sf_g->bdi.ra_pages = 0; /* No readahead */
-# if LINUX_VERSION_CODE >= KERNEL_VERSION (2, 6, 12)
+# if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 12)
     sf_g->bdi.capabilities  = BDI_CAP_MAP_DIRECT    /* MAP_SHARED */
                             | BDI_CAP_MAP_COPY      /* MAP_PRIVATE */
                             | BDI_CAP_READ_MAP      /* can be mapped for reading */
                             | BDI_CAP_WRITE_MAP     /* can be mapped for writing */
                             | BDI_CAP_EXEC_MAP;     /* can be mapped for execution */
 # endif /* >= 2.6.12 */
-# if LINUX_VERSION_CODE >= KERNEL_VERSION (2, 6, 24)
+# if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 24)
     rc = bdi_init(&sf_g->bdi);
-#  if LINUX_VERSION_CODE >= KERNEL_VERSION (2, 6, 26)
+#  if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 26)
     if (!rc)
         rc = bdi_register(&sf_g->bdi, NULL, "vboxsf-%s", name);
 #  endif /* >= 2.6.26 */
@@ -821,7 +825,7 @@ int sf_init_backing_dev(struct sf_glob_info *sf_g, const char *name)
 
 void sf_done_backing_dev(struct sf_glob_info *sf_g)
 {
-#if LINUX_VERSION_CODE >= KERNEL_VERSION (2, 6, 24)
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 24)
     bdi_destroy(&sf_g->bdi); /* includes bdi_unregister() */
 #endif
 }
