@@ -1,11 +1,10 @@
+/* $Id: DrvRawImage.cpp $ */
 /** @file
- *
- * VBox storage devices:
- * Raw image driver
+ * VBox storage devices: Raw image driver
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2010 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -14,10 +13,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 
@@ -29,6 +24,7 @@
 #include <iprt/assert.h>
 #include <iprt/file.h>
 #include <iprt/string.h>
+#include <iprt/uuid.h>
 
 #include "Builtins.h"
 
@@ -36,7 +32,6 @@
 /*******************************************************************************
 *   Defined Constants And Macros                                               *
 *******************************************************************************/
-
 /** Converts a pointer to RAWIMAGE::IMedia to a PRDVRAWIMAGE. */
 #define PDMIMEDIA_2_DRVRAWIMAGE(pInterface) ( (PDRVRAWIMAGE)((uintptr_t)pInterface - RT_OFFSETOF(DRVRAWIMAGE, IMedia)) )
 
@@ -53,6 +48,8 @@
 *******************************************************************************/
 /**
  * Block driver instance data.
+ *
+ * @implements  PDMIMEDIA
  */
 typedef struct DRVRAWIMAGE
 {
@@ -70,121 +67,7 @@ typedef struct DRVRAWIMAGE
 
 
 
-/*******************************************************************************
-*   Internal Functions                                                         *
-*******************************************************************************/
-static DECLCALLBACK(int) drvRawImageRead(PPDMIMEDIA pInterface, uint64_t off, void *pvBuf, size_t cbRead);
-static DECLCALLBACK(int) drvRawImageWrite(PPDMIMEDIA pInterface, uint64_t off, const void *pvBuf, size_t cbWrite);
-static DECLCALLBACK(int) drvRawImageFlush(PPDMIMEDIA pInterface);
-static DECLCALLBACK(bool) drvRawImageIsReadOnly(PPDMIMEDIA pInterface);
-static DECLCALLBACK(uint64_t) drvRawImageGetSize(PPDMIMEDIA pInterface);
-static DECLCALLBACK(int) drvRawImageGetUuid(PPDMIMEDIA pInterface, PRTUUID pUuid);
-static DECLCALLBACK(int) drvRawImageBiosGetPCHSGeometry(PPDMIMEDIA pInterface, PPDMMEDIAGEOMETRY pPCHSGeometry);
-static DECLCALLBACK(int) drvRawImageBiosSetPCHSGeometry(PPDMIMEDIA pInterface, PCPDMMEDIAGEOMETRY pPCHSGeometry);
-static DECLCALLBACK(int) drvRawImageBiosGetLCHSGeometry(PPDMIMEDIA pInterface, PPDMMEDIAGEOMETRY pLCHSGeometry);
-static DECLCALLBACK(int) drvRawImageBiosSetLCHSGeometry(PPDMIMEDIA pInterface, PCPDMMEDIAGEOMETRY pLCHSGeometry);
-
-static DECLCALLBACK(void *) drvRawImageQueryInterface(PPDMIBASE pInterface, PDMINTERFACE enmInterface);
-
-
-
-
-/**
- * Construct a raw image driver instance.
- *  
- * @copydoc FNPDMDRVCONSTRUCT
- */ 
-static DECLCALLBACK(int) drvRawImageConstruct(PPDMDRVINS pDrvIns, PCFGMNODE pCfgHandle, uint32_t fFlags)
-{
-    PDRVRAWIMAGE pThis = PDMINS_2_DATA(pDrvIns, PDRVRAWIMAGE);
-
-    /*
-     * Init the static parts.
-     */
-    pThis->pDrvIns                      = pDrvIns;
-    pThis->File                         = NIL_RTFILE;
-    /* IBase */
-    pDrvIns->IBase.pfnQueryInterface    = drvRawImageQueryInterface;
-    /* IMedia */
-    pThis->IMedia.pfnRead               = drvRawImageRead;
-    pThis->IMedia.pfnWrite              = drvRawImageWrite;
-    pThis->IMedia.pfnFlush              = drvRawImageFlush;
-    pThis->IMedia.pfnGetSize            = drvRawImageGetSize;
-    pThis->IMedia.pfnGetUuid            = drvRawImageGetUuid;
-    pThis->IMedia.pfnIsReadOnly         = drvRawImageIsReadOnly;
-    pThis->IMedia.pfnBiosGetPCHSGeometry = drvRawImageBiosGetPCHSGeometry;
-    pThis->IMedia.pfnBiosSetPCHSGeometry = drvRawImageBiosSetPCHSGeometry;
-    pThis->IMedia.pfnBiosGetLCHSGeometry = drvRawImageBiosGetLCHSGeometry;
-    pThis->IMedia.pfnBiosSetLCHSGeometry = drvRawImageBiosSetLCHSGeometry;
-
-    /*
-     * Read the configuration.
-     */
-    if (!CFGMR3AreValuesValid(pCfgHandle, "Path\0"))
-        return VERR_PDM_DRVINS_UNKNOWN_CFG_VALUES;
-
-    char *pszName;
-    int rc = CFGMR3QueryStringAlloc(pCfgHandle, "Path", &pszName);
-    if (RT_FAILURE(rc))
-    {
-        AssertMsgFailed(("Configuration error: query for \"Path\" string return %Rrc.\n", rc));
-        return rc;
-    }
-
-    /*
-     * Open the image.
-     */
-    rc = RTFileOpen(&pThis->File, pszName,
-                    RTFILE_O_READWRITE | RTFILE_O_OPEN | RTFILE_O_DENY_NONE);
-    if (RT_SUCCESS(rc))
-    {
-        LogFlow(("drvRawImageConstruct: Raw image '%s' opened successfully.\n", pszName));
-        pThis->pszFilename = pszName;
-        pThis->fReadOnly = false;
-    }
-    else
-    {
-        rc = RTFileOpen(&pThis->File, pszName,
-                        RTFILE_O_READ | RTFILE_O_OPEN | RTFILE_O_DENY_NONE);
-        if (RT_SUCCESS(rc))
-        {
-            LogFlow(("drvRawImageConstruct: Raw image '%s' opened successfully.\n", pszName));
-            pThis->pszFilename = pszName;
-            pThis->fReadOnly = true;
-        }
-        else
-        {
-            AssertMsgFailed(("Could not open Raw image file %s, rc=%Rrc\n", pszName, rc));
-            MMR3HeapFree(pszName);
-        }
-    }
-
-    return rc;
-}
-
-
-/**
- * Destruct a driver instance.
- *
- * Most VM resources are freed by the VM. This callback is provided so that any non-VM
- * resources can be freed correctly.
- *
- * @param   pDrvIns     The driver instance data.
- */
-static DECLCALLBACK(void) drvRawImageDestruct(PPDMDRVINS pDrvIns)
-{
-    PDRVRAWIMAGE pThis = PDMINS_2_DATA(pDrvIns, PDRVRAWIMAGE);
-    LogFlow(("drvRawImageDestruct: '%s'\n", pThis->pszFilename));
-
-    if (pThis->File != NIL_RTFILE)
-    {
-        RTFileClose(pThis->File);
-        pThis->File = NIL_RTFILE;
-    }
-    if (pThis->pszFilename)
-        MMR3HeapFree(pThis->pszFilename);
-}
-
+/* -=-=-=-=- PDMIMEDIA -=-=-=-=- */
 
 /** @copydoc PDMIMEDIA::pfnGetSize */
 static DECLCALLBACK(uint64_t) drvRawImageGetSize(PPDMIMEDIA pInterface)
@@ -334,28 +217,119 @@ static DECLCALLBACK(bool) drvRawImageIsReadOnly(PPDMIMEDIA pInterface)
 }
 
 
+/* -=-=-=-=- PDMIBASE -=-=-=-=- */
+
 /**
- * Queries an interface to the driver.
- *
- * @returns Pointer to interface.
- * @returns NULL if the interface was not supported by the driver.
- * @param   pInterface          Pointer to this interface structure.
- * @param   enmInterface        The requested interface identification.
- * @thread  Any thread.
+ * @interface_method_impl{PDMIBASE,pfnQueryInterface}
  */
-static DECLCALLBACK(void *) drvRawImageQueryInterface(PPDMIBASE pInterface, PDMINTERFACE enmInterface)
+static DECLCALLBACK(void *) drvRawImageQueryInterface(PPDMIBASE pInterface, const char *pszIID)
+{
+    PPDMDRVINS      pDrvIns = PDMIBASE_2_DRVINS(pInterface);
+    PDRVRAWIMAGE    pThis   = PDMINS_2_DATA(pDrvIns, PDRVRAWIMAGE);
+
+    PDMIBASE_RETURN_INTERFACE(pszIID, PDMIBASE, &pDrvIns->IBase);
+    PDMIBASE_RETURN_INTERFACE(pszIID, PDMIMEDIA, &pThis->IMedia);
+    return NULL;
+}
+
+/* -=-=-=-=- PDMDRVREG -=-=-=-=- */
+
+/**
+ * Destruct a driver instance.
+ *
+ * Most VM resources are freed by the VM. This callback is provided so that any non-VM
+ * resources can be freed correctly.
+ *
+ * @param   pDrvIns     The driver instance data.
+ */
+static DECLCALLBACK(void) drvRawImageDestruct(PPDMDRVINS pDrvIns)
 {
-    PPDMDRVINS pDrvIns = PDMIBASE_2_DRVINS(pInterface);
     PDRVRAWIMAGE pThis = PDMINS_2_DATA(pDrvIns, PDRVRAWIMAGE);
-    switch (enmInterface)
+    LogFlow(("drvRawImageDestruct: '%s'\n", pThis->pszFilename));
+    PDMDRV_CHECK_VERSIONS_RETURN_VOID(pDrvIns);
+
+    if (pThis->File != NIL_RTFILE)
     {
-        case PDMINTERFACE_BASE:
-            return &pDrvIns->IBase;
-        case PDMINTERFACE_MEDIA:
-            return &pThis->IMedia;
-        default:
-            return NULL;
+        RTFileClose(pThis->File);
+        pThis->File = NIL_RTFILE;
     }
+    if (pThis->pszFilename)
+        MMR3HeapFree(pThis->pszFilename);
+}
+
+
+/**
+ * Construct a raw image driver instance.
+ *
+ * @copydoc FNPDMDRVCONSTRUCT
+ */
+static DECLCALLBACK(int) drvRawImageConstruct(PPDMDRVINS pDrvIns, PCFGMNODE pCfg, uint32_t fFlags)
+{
+    PDRVRAWIMAGE pThis = PDMINS_2_DATA(pDrvIns, PDRVRAWIMAGE);
+    PDMDRV_CHECK_VERSIONS_RETURN(pDrvIns);
+
+    /*
+     * Init the static parts.
+     */
+    pThis->pDrvIns                      = pDrvIns;
+    pThis->File                         = NIL_RTFILE;
+    /* IBase */
+    pDrvIns->IBase.pfnQueryInterface    = drvRawImageQueryInterface;
+    /* IMedia */
+    pThis->IMedia.pfnRead               = drvRawImageRead;
+    pThis->IMedia.pfnWrite              = drvRawImageWrite;
+    pThis->IMedia.pfnFlush              = drvRawImageFlush;
+    pThis->IMedia.pfnGetSize            = drvRawImageGetSize;
+    pThis->IMedia.pfnGetUuid            = drvRawImageGetUuid;
+    pThis->IMedia.pfnIsReadOnly         = drvRawImageIsReadOnly;
+    pThis->IMedia.pfnBiosGetPCHSGeometry = drvRawImageBiosGetPCHSGeometry;
+    pThis->IMedia.pfnBiosSetPCHSGeometry = drvRawImageBiosSetPCHSGeometry;
+    pThis->IMedia.pfnBiosGetLCHSGeometry = drvRawImageBiosGetLCHSGeometry;
+    pThis->IMedia.pfnBiosSetLCHSGeometry = drvRawImageBiosSetLCHSGeometry;
+
+    /*
+     * Read the configuration.
+     */
+    if (!CFGMR3AreValuesValid(pCfg, "Path\0"))
+        return VERR_PDM_DRVINS_UNKNOWN_CFG_VALUES;
+
+    char *pszName;
+    int rc = CFGMR3QueryStringAlloc(pCfg, "Path", &pszName);
+    if (RT_FAILURE(rc))
+    {
+        AssertMsgFailed(("Configuration error: query for \"Path\" string return %Rrc.\n", rc));
+        return rc;
+    }
+
+    /*
+     * Open the image.
+     */
+    rc = RTFileOpen(&pThis->File, pszName,
+                    RTFILE_O_READWRITE | RTFILE_O_OPEN | RTFILE_O_DENY_NONE);
+    if (RT_SUCCESS(rc))
+    {
+        LogFlow(("drvRawImageConstruct: Raw image '%s' opened successfully.\n", pszName));
+        pThis->pszFilename = pszName;
+        pThis->fReadOnly = false;
+    }
+    else
+    {
+        rc = RTFileOpen(&pThis->File, pszName,
+                        RTFILE_O_READ | RTFILE_O_OPEN | RTFILE_O_DENY_NONE);
+        if (RT_SUCCESS(rc))
+        {
+            LogFlow(("drvRawImageConstruct: Raw image '%s' opened successfully.\n", pszName));
+            pThis->pszFilename = pszName;
+            pThis->fReadOnly = true;
+        }
+        else
+        {
+            AssertMsgFailed(("Could not open Raw image file %s, rc=%Rrc\n", pszName, rc));
+            MMR3HeapFree(pszName);
+        }
+    }
+
+    return rc;
 }
 
 
@@ -366,8 +340,12 @@ const PDMDRVREG g_DrvRawImage =
 {
     /* u32Version */
     PDM_DRVREG_VERSION,
-    /* szDriverName */
+    /* szName */
     "RawImage",
+    /* szRCMod */
+    "",
+    /* szR0Mod */
+    "",
     /* pszDescription */
     "Raw image access driver.",
     /* fFlags */
@@ -382,6 +360,8 @@ const PDMDRVREG g_DrvRawImage =
     drvRawImageConstruct,
     /* pfnDestruct */
     drvRawImageDestruct,
+    /* pfnRelocate */
+    NULL,
     /* pfnIOCtl */
     NULL,
     /* pfnPowerOn */
@@ -395,9 +375,9 @@ const PDMDRVREG g_DrvRawImage =
     /* pfnAttach */
     NULL,
     /* pfnDetach */
-    NULL, 
+    NULL,
     /* pfnPowerOff */
-    NULL, 
+    NULL,
     /* pfnSoftReset */
     NULL,
     /* u32EndVersion */
