@@ -158,6 +158,8 @@ typedef struct VBOXDISK
     PPDMIMEDIAASYNCPORT      pDrvMediaAsyncPort;
     /** Pointer to the list of data we need to keep per image. */
     PVBOXIMAGE               pImages;
+    /** Flag whether the media should allow concurrent open for writing. */
+    bool                fShareable;
     /** Flag whether a merge operation has been set up. */
     bool                fMergePending;
     /** Synchronization to prevent destruction before merge finishes. */
@@ -333,10 +335,19 @@ static DECLCALLBACK(int) drvvdAsyncIOOpen(void *pvUser, const char *pszLocation,
                                                         drvvdAsyncTaskCompleted, pStorageBackend, "AsyncTaskCompleted");
             if (RT_SUCCESS(rc))
             {
-                rc = PDMR3AsyncCompletionEpCreateForFile(&pStorageBackend->pEndpoint, pszLocation,
-                                                         uOpenFlags & VD_INTERFACEASYNCIO_OPEN_FLAGS_READONLY
-                                                         ? PDMACEP_FILE_FLAGS_READ_ONLY | PDMACEP_FILE_FLAGS_CACHING
-                                                         : PDMACEP_FILE_FLAGS_CACHING,
+                uint32_t fFlags =    uOpenFlags & VD_INTERFACEASYNCIO_OPEN_FLAGS_READONLY
+                                   ? PDMACEP_FILE_FLAGS_READ_ONLY | PDMACEP_FILE_FLAGS_CACHING
+                                   : 0;
+                if (pThis->fShareable)
+                {
+                    Assert(uOpenFlags & VD_INTERFACEASYNCIO_OPEN_FLAGS_DONT_LOCK);
+
+                    fFlags |= PDMACEP_FILE_FLAGS_DONT_LOCK;
+                }
+                else
+                    fFlags |= PDMACEP_FILE_FLAGS_CACHING;
+                rc = PDMR3AsyncCompletionEpCreateForFile(&pStorageBackend->pEndpoint,
+                                                         pszLocation, fFlags,
                                                          pStorageBackend->pTemplate);
                 if (RT_SUCCESS(rc))
                 {
@@ -585,6 +596,14 @@ static int drvvdCfgQuery(void *pvUser, const char *pszName, char *pszString, siz
 *   VD TCP network stack interface implementation - INIP case                  *
 *******************************************************************************/
 
+typedef union INIPSOCKADDRUNION
+{
+    struct sockaddr     Addr;
+    struct sockaddr_in  Ipv4;
+} INIPSOCKADDRUNION;
+
+static DECLCALLBACK(int) drvvdINIPFlush(RTSOCKET Sock);
+
 /** @copydoc VDINTERFACETCPNET::pfnClientConnect */
 static DECLCALLBACK(int) drvvdINIPClientConnect(const char *pszAddress, uint32_t uPort, PRTSOCKET pSock)
 {
@@ -604,7 +623,7 @@ static DECLCALLBACK(int) drvvdINIPClientConnect(const char *pszAddress, uint32_t
         return VERR_NET_HOST_UNREACHABLE;
     }
     /* Create socket and connect. */
-    RTSOCKET Sock = lwip_socket(PF_INET, SOCK_STREAM, 0);
+    int Sock = lwip_socket(PF_INET, SOCK_STREAM, 0);
     if (Sock != -1)
     {
         struct sockaddr_in InAddr = {0};
@@ -613,7 +632,7 @@ static DECLCALLBACK(int) drvvdINIPClientConnect(const char *pszAddress, uint32_t
         InAddr.sin_addr = ip;
         if (!lwip_connect(Sock, (struct sockaddr *)&InAddr, sizeof(InAddr)))
         {
-            *pSock = Sock;
+            *pSock = (RTSOCKET)Sock;
             return VINF_SUCCESS;
         }
         rc = VERR_NET_CONNECTION_REFUSED; /* @todo real solution needed */
@@ -627,7 +646,7 @@ static DECLCALLBACK(int) drvvdINIPClientConnect(const char *pszAddress, uint32_t
 /** @copydoc VDINTERFACETCPNET::pfnClientClose */
 static DECLCALLBACK(int) drvvdINIPClientClose(RTSOCKET Sock)
 {
-    lwip_close(Sock);
+    lwip_close((uintptr_t)Sock);
     return VINF_SUCCESS; /** @todo real solution needed */
 }
 
@@ -636,18 +655,18 @@ static DECLCALLBACK(int) drvvdINIPSelectOne(RTSOCKET Sock, RTMSINTERVAL cMillies
 {
     fd_set fdsetR;
     FD_ZERO(&fdsetR);
-    FD_SET(Sock, &fdsetR);
+    FD_SET((uintptr_t)Sock, &fdsetR);
     fd_set fdsetE = fdsetR;
 
     int rc;
     if (cMillies == RT_INDEFINITE_WAIT)
-        rc = lwip_select(Sock + 1, &fdsetR, NULL, &fdsetE, NULL);
+        rc = lwip_select((uintptr_t)Sock + 1, &fdsetR, NULL, &fdsetE, NULL);
     else
     {
         struct timeval timeout;
         timeout.tv_sec = cMillies / 1000;
         timeout.tv_usec = (cMillies % 1000) * 1000;
-        rc = lwip_select(Sock + 1, &fdsetR, NULL, &fdsetE, &timeout);
+        rc = lwip_select((uintptr_t)Sock + 1, &fdsetR, NULL, &fdsetE, &timeout);
     }
     if (rc > 0)
         return VINF_SUCCESS;
@@ -677,7 +696,7 @@ static DECLCALLBACK(int) drvvdINIPRead(RTSOCKET Sock, void *pvBuffer, size_t cbB
         /** @todo this clipping here is just in case (the send function
          * needed it, so I added it here, too). Didn't investigate if this
          * really has issues. Better be safe than sorry. */
-        ssize_t cbBytesRead = lwip_recv(Sock, (char *)pvBuffer + cbRead,
+        ssize_t cbBytesRead = lwip_recv((uintptr_t)Sock, (char *)pvBuffer + cbRead,
                                         RT_MIN(cbToRead, 32768), 0);
         if (cbBytesRead < 0)
             return VERR_NET_CONNECTION_REFUSED; /** @todo real solution */
@@ -711,7 +730,7 @@ static DECLCALLBACK(int) drvvdINIPWrite(RTSOCKET Sock, const void *pvBuffer, siz
          * send (stupid limitation buried in the code), so make sure we
          * don't get any wraparounds. This should be moved to DevINIP
          * stack interface once that's implemented. */
-        ssize_t cbWritten = lwip_send(Sock, (void *)pvBuffer,
+        ssize_t cbWritten = lwip_send((uintptr_t)Sock, (void *)pvBuffer,
                                       RT_MIN(cbBuffer, 32768), 0);
         if (cbWritten < 0)
             return VERR_NET_CONNECTION_REFUSED; /** @todo real solution needed */
@@ -724,14 +743,43 @@ static DECLCALLBACK(int) drvvdINIPWrite(RTSOCKET Sock, const void *pvBuffer, siz
     return VINF_SUCCESS;
 }
 
+/** @copydoc VDINTERFACETCPNET::pfnSgWrite */
+static DECLCALLBACK(int) drvvdINIPSgWrite(RTSOCKET Sock, PCRTSGBUF pSgBuf)
+{
+    int rc = VINF_SUCCESS;
+
+    /* This is an extremely crude emulation, however it's good enough
+     * for our iSCSI code. INIP has no sendmsg(). */
+    for (unsigned i = 0; i < pSgBuf->cSegs; i++)
+    {
+        rc = drvvdINIPWrite(Sock, pSgBuf->paSegs[i].pvSeg,
+                            pSgBuf->paSegs[i].cbSeg);
+        if (RT_FAILURE(rc))
+            break;
+    }
+    if (RT_SUCCESS(rc))
+        drvvdINIPFlush(Sock);
+
+    return rc;
+}
+
 /** @copydoc VDINTERFACETCPNET::pfnFlush */
 static DECLCALLBACK(int) drvvdINIPFlush(RTSOCKET Sock)
 {
     int fFlag = 1;
-    lwip_setsockopt(Sock, IPPROTO_TCP, TCP_NODELAY,
+    lwip_setsockopt((uintptr_t)Sock, IPPROTO_TCP, TCP_NODELAY,
                     (const char *)&fFlag, sizeof(fFlag));
     fFlag = 0;
-    lwip_setsockopt(Sock, IPPROTO_TCP, TCP_NODELAY,
+    lwip_setsockopt((uintptr_t)Sock, IPPROTO_TCP, TCP_NODELAY,
+                    (const char *)&fFlag, sizeof(fFlag));
+    return VINF_SUCCESS;
+}
+
+/** @copydoc VDINTERFACETCPNET::pfnSetSendCoalescing */
+static DECLCALLBACK(int) drvvdINIPSetSendCoalescing(RTSOCKET Sock, bool fEnable)
+{
+    int fFlag = fEnable ? 0 : 1;
+    lwip_setsockopt((uintptr_t)Sock, IPPROTO_TCP, TCP_NODELAY,
                     (const char *)&fFlag, sizeof(fFlag));
     return VINF_SUCCESS;
 }
@@ -739,14 +787,10 @@ static DECLCALLBACK(int) drvvdINIPFlush(RTSOCKET Sock)
 /** @copydoc VDINTERFACETCPNET::pfnGetLocalAddress */
 static DECLCALLBACK(int) drvvdINIPGetLocalAddress(RTSOCKET Sock, PRTNETADDR pAddr)
 {
-   union
-    {
-        struct sockaddr     Addr;
-        struct sockaddr_in  Ipv4;
-    }               u;
-    socklen_t       cbAddr = sizeof(u);
+    INIPSOCKADDRUNION u;
+    socklen_t cbAddr = sizeof(u);
     RT_ZERO(u);
-    if (!lwip_getsockname(Sock, &u.Addr, &cbAddr))
+    if (!lwip_getsockname((uintptr_t)Sock, &u.Addr, &cbAddr))
     {
         /*
          * Convert the address.
@@ -769,14 +813,10 @@ static DECLCALLBACK(int) drvvdINIPGetLocalAddress(RTSOCKET Sock, PRTNETADDR pAdd
 /** @copydoc VDINTERFACETCPNET::pfnGetPeerAddress */
 static DECLCALLBACK(int) drvvdINIPGetPeerAddress(RTSOCKET Sock, PRTNETADDR pAddr)
 {
-   union
-    {
-        struct sockaddr     Addr;
-        struct sockaddr_in  Ipv4;
-    }               u;
-    socklen_t       cbAddr = sizeof(u);
+    INIPSOCKADDRUNION u;
+    socklen_t cbAddr = sizeof(u);
     RT_ZERO(u);
-    if (!lwip_getpeername(Sock, &u.Addr, &cbAddr))
+    if (!lwip_getpeername((uintptr_t)Sock, &u.Addr, &cbAddr))
     {
         /*
          * Convert the address.
@@ -1203,6 +1243,7 @@ static DECLCALLBACK(int) drvvdConstruct(PPDMDRVINS pDrvIns,
     char *pszName = NULL;   /**< The path of the disk image file. */
     char *pszFormat = NULL; /**< The format backed to use for this image. */
     bool fReadOnly;         /**< True if the media is read-only. */
+    bool fMaybeReadOnly;    /**< True if the media may or may not be read-only. */
     bool fHonorZeroWrites;  /**< True if zero blocks should be written. */
     PDMDRV_CHECK_VERSIONS_RETURN(pDrvIns);
 
@@ -1214,6 +1255,7 @@ static DECLCALLBACK(int) drvvdConstruct(PPDMDRVINS pDrvIns,
     pThis->fTempReadOnly                = false;
     pThis->pDisk                        = NULL;
     pThis->fAsyncIOSupported            = false;
+    pThis->fShareable                   = false;
     pThis->fMergePending                = false;
     pThis->MergeCompleteMutex           = NIL_RTSEMFASTMUTEX;
     pThis->uMergeSource                 = VD_LAST_IMAGE;
@@ -1282,7 +1324,7 @@ static DECLCALLBACK(int) drvvdConstruct(PPDMDRVINS pDrvIns,
              * open flags. Some might be converted to per-image flags later. */
             fValid = CFGMR3AreValuesValid(pCurNode,
                                           "Format\0Path\0"
-                                          "ReadOnly\0TempReadOnly\0HonorZeroWrites\0"
+                                          "ReadOnly\0MaybeReadOnly\0TempReadOnly\0Shareable\0HonorZeroWrites\0"
                                           "HostIPStack\0UseNewIo\0"
                                           "SetupMerge\0MergeSource\0MergeTarget\0");
         }
@@ -1326,6 +1368,14 @@ static DECLCALLBACK(int) drvvdConstruct(PPDMDRVINS pDrvIns,
                 break;
             }
 
+            rc = CFGMR3QueryBoolDef(pCurNode, "MaybeReadOnly", &fMaybeReadOnly, false);
+            if (RT_FAILURE(rc))
+            {
+                rc = PDMDRV_SET_ERROR(pDrvIns, rc,
+                                      N_("DrvVD: Configuration error: Querying \"MaybeReadOnly\" as boolean failed"));
+                break;
+            }
+
             rc = CFGMR3QueryBoolDef(pCurNode, "TempReadOnly", &pThis->fTempReadOnly, false);
             if (RT_FAILURE(rc))
             {
@@ -1339,6 +1389,15 @@ static DECLCALLBACK(int) drvvdConstruct(PPDMDRVINS pDrvIns,
                                       N_("DrvVD: Configuration error: Both \"ReadOnly\" and \"TempReadOnly\" are set"));
                 break;
             }
+
+            rc = CFGMR3QueryBoolDef(pCurNode, "Shareable", &pThis->fShareable, false);
+            if (RT_FAILURE(rc))
+            {
+                rc = PDMDRV_SET_ERROR(pDrvIns, rc,
+                                      N_("DrvVD: Configuration error: Querying \"Shareable\" as boolean failed"));
+                break;
+            }
+
             rc = CFGMR3QueryBoolDef(pCurNode, "UseNewIo", &fUseNewIo, false);
             if (RT_FAILURE(rc))
             {
@@ -1385,7 +1444,9 @@ static DECLCALLBACK(int) drvvdConstruct(PPDMDRVINS pDrvIns,
             pThis->VDITcpNetCallbacks.pfnSelectOne = RTTcpSelectOne;
             pThis->VDITcpNetCallbacks.pfnRead = RTTcpRead;
             pThis->VDITcpNetCallbacks.pfnWrite = RTTcpWrite;
+            pThis->VDITcpNetCallbacks.pfnSgWrite = RTTcpSgWrite;
             pThis->VDITcpNetCallbacks.pfnFlush = RTTcpFlush;
+            pThis->VDITcpNetCallbacks.pfnSetSendCoalescing = RTTcpSetSendCoalescing;
             pThis->VDITcpNetCallbacks.pfnGetLocalAddress = RTTcpGetLocalAddress;
             pThis->VDITcpNetCallbacks.pfnGetPeerAddress = RTTcpGetPeerAddress;
         }
@@ -1402,7 +1463,9 @@ static DECLCALLBACK(int) drvvdConstruct(PPDMDRVINS pDrvIns,
             pThis->VDITcpNetCallbacks.pfnSelectOne = drvvdINIPSelectOne;
             pThis->VDITcpNetCallbacks.pfnRead = drvvdINIPRead;
             pThis->VDITcpNetCallbacks.pfnWrite = drvvdINIPWrite;
+            pThis->VDITcpNetCallbacks.pfnSgWrite = drvvdINIPSgWrite;
             pThis->VDITcpNetCallbacks.pfnFlush = drvvdINIPFlush;
+            pThis->VDITcpNetCallbacks.pfnSetSendCoalescing = drvvdINIPSetSendCoalescing;
             pThis->VDITcpNetCallbacks.pfnGetLocalAddress = drvvdINIPGetLocalAddress;
             pThis->VDITcpNetCallbacks.pfnGetPeerAddress = drvvdINIPGetPeerAddress;
 #endif /* VBOX_WITH_INIP */
@@ -1566,6 +1629,8 @@ static DECLCALLBACK(int) drvvdConstruct(PPDMDRVINS pDrvIns,
             uOpenFlags |= VD_OPEN_FLAGS_HONOR_ZEROES;
         if (pThis->fAsyncIOSupported)
             uOpenFlags |= VD_OPEN_FLAGS_ASYNC_IO;
+        if (pThis->fShareable)
+            uOpenFlags |= VD_OPEN_FLAGS_SHAREABLE;
 
         /* Try to open backend in async I/O mode first. */
         rc = VDOpen(pThis->pDisk, pszFormat, pszName, uOpenFlags, pImage->pVDIfsImage);
@@ -1581,8 +1646,9 @@ static DECLCALLBACK(int) drvvdConstruct(PPDMDRVINS pDrvIns,
             Log(("%s: %d - Opened '%s' in %s mode\n", __FUNCTION__,
                  iLevel, pszName,
                  VDIsReadOnly(pThis->pDisk) ? "read-only" : "read-write"));
-            if (   VDIsReadOnly(pThis->pDisk)
+            if (  VDIsReadOnly(pThis->pDisk)
                 && !fReadOnly
+                && !fMaybeReadOnly
                 && !pThis->fTempReadOnly
                 && iLevel == 0)
             {
