@@ -81,6 +81,11 @@ static ISession *gSession = NULL;
 static IConsole *gConsole = NULL;
 static EventQueue *gEventQ = NULL;
 
+#ifdef VBOX_WITH_VNC
+static VNCFB *g_pFramebufferVNC;
+#endif
+
+
 ////////////////////////////////////////////////////////////////////////////////
 
 /**
@@ -201,8 +206,8 @@ public:
         Utf8Str utf8Key = key;
         if (utf8Key == "/VirtualBox/GuestInfo/OS/NoLoggedInUsers")
         {
-            /* Check if the "disconnect on logout feature" is enabled. */
-            BOOL fDisconnectOnGuestLogout = FALSE;
+            /* Check if this is our machine and the "disconnect on logout feature" is enabled. */
+            BOOL fProcessDisconnectOnGuestLogout = FALSE;
             ComPtr <IMachine> machine;
             HRESULT hrc = S_OK;
 
@@ -211,16 +216,21 @@ public:
                 hrc = gConsole->COMGETTER(Machine)(machine.asOutParam());
                 if (SUCCEEDED(hrc) && machine)
                 {
-                    Bstr value1;
-                    hrc = machine->GetExtraData(Bstr("VRDP/DisconnectOnGuestLogout"), value1.asOutParam());
-                    if (SUCCEEDED(hrc) && value1 == "1")
+                    Bstr id;
+                    hrc = machine->COMGETTER(Id)(id.asOutParam());
+                    if (id == machineId)
                     {
-                        fDisconnectOnGuestLogout = TRUE;
+                        Bstr value1;
+                        hrc = machine->GetExtraData(Bstr("VRDP/DisconnectOnGuestLogout"), value1.asOutParam());
+                        if (SUCCEEDED(hrc) && value1 == "1")
+                        {
+                            fProcessDisconnectOnGuestLogout = TRUE;
+                        }
                     }
                 }
             }
 
-            if (fDisconnectOnGuestLogout)
+            if (fProcessDisconnectOnGuestLogout)
             {
                 Utf8Str utf8Value = value;
                 if (utf8Value == "true")
@@ -321,6 +331,10 @@ public:
                 mouse->PutMouseEventAbsolute(-1, -1, 0, 0 /* Horizontal wheel */, 0);
             }
         }
+#ifdef VBOX_WITH_VNC
+        if (g_pFramebufferVNC)
+            g_pFramebufferVNC->enableAbsMouse(supportsAbsolute);
+#endif
         return S_OK;
     }
 
@@ -657,7 +671,7 @@ extern "C" DECLEXPORT(int) TrustedMain(int argc, char **argv, char **envp)
         OPT_NO_PATM,
         OPT_CSAM,
         OPT_NO_CSAM,
-        OPT_COMMENT,
+        OPT_COMMENT
     };
 
     static const RTGETOPTDEF s_aOptions[] =
@@ -962,18 +976,20 @@ extern "C" DECLEXPORT(int) TrustedMain(int argc, char **argv, char **envp)
 #ifdef VBOX_WITH_VNC
         if (fVNCEnable)
         {
-            VNCFB *pFramebufferVNC = new VNCFB(console, uVNCPort, pszVNCPassword);
-            rc = pFramebufferVNC->init();
+            Bstr name;
+            machine->COMGETTER(Name)(name.asOutParam());
+            g_pFramebufferVNC = new VNCFB(console, uVNCPort, pszVNCPassword);
+            rc = g_pFramebufferVNC->init(name ? Utf8Str(name).raw() : "");
             if (rc != S_OK)
             {
                 LogError("Failed to load the vnc server extension, possibly due to a damaged file\n", rc);
-                delete pFramebufferVNC;
+                delete g_pFramebufferVNC;
                 break;
             }
 
             Log2(("VBoxHeadless: Registering VNC framebuffer\n"));
-            pFramebufferVNC->AddRef();
-            display->SetFramebuffer(VBOX_VIDEO_PRIMARY_SCREEN, pFramebufferVNC);
+            g_pFramebufferVNC->AddRef();
+            display->SetFramebuffer(VBOX_VIDEO_PRIMARY_SCREEN, g_pFramebufferVNC);
         }
         if (rc != S_OK)
             break;
@@ -985,13 +1001,20 @@ extern "C" DECLEXPORT(int) TrustedMain(int argc, char **argv, char **envp)
         unsigned uScreenId;
         for (uScreenId = 0; uScreenId < cMonitors; uScreenId++)
         {
-#ifdef VBOX_FFMPEG
+# ifdef VBOX_FFMPEG
             if (fFFMPEG && uScreenId == 0)
             {
                 /* Already registered. */
                 continue;
             }
-#endif /* defined(VBOX_FFMPEG) */
+# endif
+# ifdef VBOX_WITH_VNC
+            if (fVNCEnable && uScreenId == 0)
+            {
+                /* Already registered. */
+                continue;
+            }
+# endif
             VRDPFramebuffer *pVRDPFramebuffer = new VRDPFramebuffer();
             if (!pVRDPFramebuffer)
             {
@@ -1128,6 +1151,23 @@ extern "C" DECLEXPORT(int) TrustedMain(int argc, char **argv, char **envp)
         CHECK_ERROR_BREAK(console, PowerUp(progress.asOutParam()));
 
         /* wait for result because there can be errors */
+        /** @todo The error handling here is kind of peculiar, anyone care
+         *        to comment why this works just fine? */
+        for (;;)
+        {
+            rc = progress->WaitForCompletion(500);
+            if (FAILED(rc))
+                break;
+
+            /* Processing events is vital for teleportation targets. */
+            gEventQ->processEventQueue(0);
+
+            BOOL fCompleted;
+            rc = progress->COMGETTER(Completed)(&fCompleted);
+            if (FAILED(rc) || fCompleted)
+                break;
+        }
+
         if (SUCCEEDED(progress->WaitForCompletion(-1)))
         {
             LONG progressRc;
