@@ -227,7 +227,9 @@ int handleCreateHardDisk(HandlerArg *a)
             case 't':   // --type
                 vrc = parseDiskType(ValueUnion.psz, &DiskType);
                 if (    RT_FAILURE(vrc)
-                    ||  (DiskType != MediumType_Normal && DiskType != MediumType_Writethrough))
+                    ||  (   DiskType != MediumType_Normal
+                         && DiskType != MediumType_Writethrough
+                         && DiskType != MediumType_Shareable))
                     return errorArgument("Invalid hard disk type '%s'", ValueUnion.psz);
                 break;
 
@@ -303,9 +305,10 @@ int handleCreateHardDisk(HandlerArg *a)
                 Bstr uuid;
                 CHECK_ERROR(hardDisk, COMGETTER(Id)(uuid.asOutParam()));
 
-                if (DiskType == MediumType_Writethrough)
+                if (   DiskType == MediumType_Writethrough
+                    || DiskType == MediumType_Shareable)
                 {
-                    CHECK_ERROR(hardDisk, COMSETTER(Type)(MediumType_Writethrough));
+                    CHECK_ERROR(hardDisk, COMSETTER(Type)(DiskType));
                 }
 
                 RTPrintf("Disk image created. UUID: %s\n", Utf8Str(uuid).raw());
