@@ -1,10 +1,10 @@
-/* $Rev: 58763 $ */
+/* $Rev: 60692 $ */
 /** @file
  * VBoxDrv - The VirtualBox Support Driver - Linux specifics.
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -22,10 +22,6 @@
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  * Some lines of code to disable the local APIC on x86_64 machines taken
  * from a Mandriva patch by Gwenole Beauchesne <gbeauchesne@mandriva.com>.
  */
@@ -37,6 +33,7 @@
 #include "../SUPDrvInternal.h"
 #include "the-linux-kernel.h"
 #include "version-generated.h"
+#include "product-generated.h"
 
 #include <iprt/assert.h>
 #include <iprt/spinlock.h>
@@ -72,6 +69,16 @@
 #include <iprt/mem.h>
 
 
+/*******************************************************************************
+*   Defined Constants And Macros                                               *
+*******************************************************************************/
+/* check kernel version */
+# ifndef SUPDRV_AGNOSTIC
+#  if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 0)
+#   error Unsupported kernel version!
+#  endif
+# endif
+
 /* devfs defines */
 #if defined(CONFIG_DEVFS_FS) && !defined(CONFIG_VBOXDRV_AS_MISC)
 # ifdef VBOX_WITH_HARDENING
@@ -126,6 +133,33 @@ extern int nmi_active;
 #endif /* CONFIG_X86_LOCAL_APIC */
 
 
+/*******************************************************************************
+*   Internal Functions                                                         *
+*******************************************************************************/
+static int  VBoxDrvLinuxInit(void);
+static void VBoxDrvLinuxUnload(void);
+static int  VBoxDrvLinuxCreate(struct inode *pInode, struct file *pFilp);
+static int  VBoxDrvLinuxClose(struct inode *pInode, struct file *pFilp);
+#ifdef HAVE_UNLOCKED_IOCTL
+static long VBoxDrvLinuxIOCtl(struct file *pFilp, unsigned int uCmd, unsigned long ulArg);
+#else
+static int  VBoxDrvLinuxIOCtl(struct inode *pInode, struct file *pFilp, unsigned int uCmd, unsigned long ulArg);
+#endif
+static int  VBoxDrvLinuxIOCtlSlow(struct file *pFilp, unsigned int uCmd, unsigned long ulArg);
+static int  VBoxDrvLinuxErr2LinuxErr(int);
+#ifdef VBOX_WITH_SUSPEND_NOTIFICATION
+static int  VBoxDrvProbe(struct platform_device *pDev);
+# if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 30)
+static int  VBoxDrvSuspend(struct device *pDev);
+static int  VBoxDrvResume(struct device *pDev);
+# else
+static int  VBoxDrvSuspend(struct platform_device *pDev, pm_message_t State);
+static int  VBoxDrvResume(struct platform_device *pDev);
+# endif
+static void VBoxDevRelease(struct device *pDev);
+#endif
+
+
 /*******************************************************************************
 *   Global Variables                                                           *
 *******************************************************************************/
@@ -163,33 +197,6 @@ __asm__(".section execmemory, \"awx\", @progbits\n\t"
         ".text\n\t");
 #endif
 
-
-/*******************************************************************************
-*   Internal Functions                                                         *
-*******************************************************************************/
-static int  VBoxDrvLinuxInit(void);
-static void VBoxDrvLinuxUnload(void);
-static int  VBoxDrvLinuxCreate(struct inode *pInode, struct file *pFilp);
-static int  VBoxDrvLinuxClose(struct inode *pInode, struct file *pFilp);
-#ifdef HAVE_UNLOCKED_IOCTL
-static long VBoxDrvLinuxIOCtl(struct file *pFilp, unsigned int uCmd, unsigned long ulArg);
-#else
-static int  VBoxDrvLinuxIOCtl(struct inode *pInode, struct file *pFilp, unsigned int uCmd, unsigned long ulArg);
-#endif
-static int  VBoxDrvLinuxIOCtlSlow(struct file *pFilp, unsigned int uCmd, unsigned long ulArg);
-static int  VBoxDrvLinuxErr2LinuxErr(int);
-#ifdef VBOX_WITH_SUSPEND_NOTIFICATION
-static int  VBoxDrvProbe(struct platform_device *pDev);
-# if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 30)
-static int  VBoxDrvSuspend(struct device *pDev);
-static int  VBoxDrvResume(struct device *pDev);
-# else
-static int  VBoxDrvSuspend(struct platform_device *pDev, pm_message_t State);
-static int  VBoxDrvResume(struct platform_device *pDev);
-# endif
-static void VBoxDevRelease(struct device *pDev);
-#endif
-
 /** The file_operations structure. */
 static struct file_operations gFileOpsVBoxDrv =
 {
@@ -256,6 +263,9 @@ static struct platform_device gPlatformDevice =
 #endif /* VBOX_WITH_SUSPEND_NOTIFICATION */
 
 
+
+
+
 #ifdef CONFIG_X86_LOCAL_APIC
 # ifdef DO_DISABLE_NMI
 /** Stop AMD NMI watchdog (x86_64 only). */
@@ -397,8 +407,6 @@ static int __init VBoxDrvLinuxInit(void)
 {
     int       rc;
 
-    dprintf(("VBoxDrv::ModuleInit\n"));
-
 #ifdef CONFIG_X86_LOCAL_APIC
     /*
      * If an NMI occurs while we are inside the world switcher the macine will crash.
@@ -556,7 +564,7 @@ no_error:
     rc = register_chrdev((dev_t)g_iModuleMajor, DEVICE_NAME, &gFileOpsVBoxDrv);
     if (rc < 0)
     {
-        dprintf(("register_chrdev() failed with rc=%#x!\n", rc));
+        Log(("register_chrdev() failed with rc=%#x!\n", rc));
         return rc;
     }
 
@@ -575,7 +583,7 @@ no_error:
      */
     if (devfs_mk_cdev(MKDEV(DEVICE_MAJOR, 0), S_IFCHR | VBOX_DEV_FMASK, DEVICE_NAME) != 0)
     {
-        dprintf(("devfs_register failed!\n"));
+        Log(("devfs_register failed!\n"));
         rc = -EINVAL;
     }
 # endif
@@ -593,11 +601,13 @@ no_error:
             rc = RTR0MemExecDonate(&g_abExecMemory[0], sizeof(g_abExecMemory));
             printk("VBoxDrv: dbg - g_abExecMemory=%p\n", (void *)&g_abExecMemory[0]);
 #endif
+            Log(("VBoxDrv::ModuleInit\n"));
+
             /*
              * Initialize the device extension.
              */
             if (RT_SUCCESS(rc))
-                rc = supdrvInitDevExt(&g_DevExt);
+                rc = supdrvInitDevExt(&g_DevExt, sizeof(SUPDRVSESSION));
             if (RT_SUCCESS(rc))
             {
 #ifdef VBOX_WITH_SUSPEND_NOTIFICATION
@@ -643,10 +653,10 @@ no_error:
     }
 #ifdef CONFIG_VBOXDRV_AS_MISC
     misc_deregister(&gMiscDevice);
-    dprintf(("VBoxDrv::ModuleInit returning %#x (minor:%d)\n", rc, gMiscDevice.minor));
+    Log(("VBoxDrv::ModuleInit returning %#x (minor:%d)\n", rc, gMiscDevice.minor));
 #else
     unregister_chrdev(g_iModuleMajor, DEVICE_NAME);
-    dprintf(("VBoxDrv::ModuleInit returning %#x (major:%d)\n", rc, g_iModuleMajor));
+    Log(("VBoxDrv::ModuleInit returning %#x (major:%d)\n", rc, g_iModuleMajor));
 #endif
     return rc;
 }
@@ -658,7 +668,7 @@ no_error:
 static void __exit VBoxDrvLinuxUnload(void)
 {
     int                 rc;
-    dprintf(("VBoxDrvLinuxUnload\n"));
+    Log(("VBoxDrvLinuxUnload\n"));
     NOREF(rc);
 
 #ifdef VBOX_WITH_SUSPEND_NOTIFICATION
@@ -674,7 +684,7 @@ static void __exit VBoxDrvLinuxUnload(void)
     rc = misc_deregister(&gMiscDevice);
     if (rc < 0)
     {
-        dprintf(("misc_deregister failed with rc=%#x\n", rc));
+        Log(("misc_deregister failed with rc=%#x\n", rc));
     }
 #else  /* !CONFIG_VBOXDRV_AS_MISC */
 # ifdef CONFIG_DEVFS_FS
@@ -720,7 +730,7 @@ static int VBoxDrvLinuxCreate(struct inode *pInode, struct file *pFilp)
     /*
      * Call common code for the rest.
      */
-    rc = supdrvCreateSession(&g_DevExt, true /* fUser */, (PSUPDRVSESSION *)&pSession);
+    rc = supdrvCreateSession(&g_DevExt, true /* fUser */, &pSession);
     if (!rc)
     {
         pSession->Uid = vboxdrvLinuxUid();
@@ -1011,27 +1021,54 @@ bool VBOXCALL supdrvOSGetForcedAsyncTscMode(PSUPDRVDEVEXT pDevExt)
 }
 
 
+int  VBOXCALL   supdrvOSLdrOpen(PSUPDRVDEVEXT pDevExt, PSUPDRVLDRIMAGE pImage, const char *pszFilename)
+{
+    NOREF(pDevExt); NOREF(pImage); NOREF(pszFilename);
+    return VERR_NOT_SUPPORTED;
+}
+
+
+int  VBOXCALL   supdrvOSLdrValidatePointer(PSUPDRVDEVEXT pDevExt, PSUPDRVLDRIMAGE pImage, void *pv, const uint8_t *pbImageBits)
+{
+    NOREF(pDevExt); NOREF(pImage); NOREF(pv); NOREF(pbImageBits);
+    return VERR_NOT_SUPPORTED;
+}
+
+
+int  VBOXCALL   supdrvOSLdrLoad(PSUPDRVDEVEXT pDevExt, PSUPDRVLDRIMAGE pImage, const uint8_t *pbImageBits)
+{
+    NOREF(pDevExt); NOREF(pImage); NOREF(pbImageBits);
+    return VERR_NOT_SUPPORTED;
+}
+
+
+void VBOXCALL   supdrvOSLdrUnload(PSUPDRVDEVEXT pDevExt, PSUPDRVLDRIMAGE pImage)
+{
+    NOREF(pDevExt); NOREF(pImage);
+}
+
+
 /**
  * Converts a supdrv error code to an linux error code.
  *
  * @returns corresponding linux error code.
- * @param   rc  supdrv error code (SUPDRV_ERR_* defines).
+ * @param   rc          IPRT status code.
  */
 static int VBoxDrvLinuxErr2LinuxErr(int rc)
 {
     switch (rc)
     {
-        case 0:                             return 0;
-        case SUPDRV_ERR_GENERAL_FAILURE:    return -EACCES;
-        case SUPDRV_ERR_INVALID_PARAM:      return -EINVAL;
-        case SUPDRV_ERR_INVALID_MAGIC:      return -EILSEQ;
-        case SUPDRV_ERR_INVALID_HANDLE:     return -ENXIO;
-        case SUPDRV_ERR_INVALID_POINTER:    return -EFAULT;
-        case SUPDRV_ERR_LOCK_FAILED:        return -ENOLCK;
-        case SUPDRV_ERR_ALREADY_LOADED:     return -EEXIST;
-        case SUPDRV_ERR_PERMISSION_DENIED:  return -EPERM;
-        case SUPDRV_ERR_VERSION_MISMATCH:   return -ENOSYS;
-        case SUPDRV_ERR_IDT_FAILED:         return -1000;
+        case VINF_SUCCESS:              return 0;
+        case VERR_GENERAL_FAILURE:      return -EACCES;
+        case VERR_INVALID_PARAMETER:    return -EINVAL;
+        case VERR_INVALID_MAGIC:        return -EILSEQ;
+        case VERR_INVALID_HANDLE:       return -ENXIO;
+        case VERR_INVALID_POINTER:      return -EFAULT;
+        case VERR_LOCK_FAILED:          return -ENOLCK;
+        case VERR_ALREADY_LOADED:       return -EEXIST;
+        case VERR_PERMISSION_DENIED:    return -EPERM;
+        case VERR_VERSION_MISMATCH:     return -ENOSYS;
+        case VERR_IDT_FAILED:           return -1000;
     }
 
     return -EPERM;
@@ -1062,8 +1099,8 @@ RTDECL(int) SUPR0Printf(const char *pszFormat, ...)
 module_init(VBoxDrvLinuxInit);
 module_exit(VBoxDrvLinuxUnload);
 
-MODULE_AUTHOR("Sun Microsystems, Inc.");
-MODULE_DESCRIPTION("VirtualBox Support Driver");
+MODULE_AUTHOR(VBOX_VENDOR);
+MODULE_DESCRIPTION(VBOX_PRODUCT " Support Driver");
 MODULE_LICENSE("GPL");
 #ifdef MODULE_VERSION
 MODULE_VERSION(VBOX_VERSION_STRING " (" RT_XSTR(SUPDRV_IOC_VERSION) ")");
