@@ -16,7 +16,7 @@
 currentzone=`zonename`
 if test "$currentzone" = "global"; then
     echo "Removing VirtualBox services and drivers..."
-    /opt/VirtualBox/vboxconfig.sh --preremove
+    ${BASEDIR}/opt/VirtualBox/vboxconfig.sh --preremove
     if test "$?" -eq 0; then
         echo "Done."
         exit 0
