@@ -1142,8 +1142,9 @@ STDMETHODIMP Host::InsertUSBDeviceFilter(ULONG aPosition,
         pFilter->getId() = m->pUSBProxyService->insertFilter(&pFilter->getData().mUSBFilter);
     }
 
-    /* save the global settings */
+    // save the global settings; for that we should hold only the VirtualBox lock
     alock.release();
+    AutoWriteLock vboxLock(m->pParent COMMA_LOCKVAL_SRC_POS);
     return rc = m->pParent->saveSettings();
 #else
     /* Note: The GUI depends on this method returning E_NOTIMPL with no
@@ -1197,8 +1198,9 @@ STDMETHODIMP Host::RemoveUSBDeviceFilter(ULONG aPosition)
         filter->getId() = NULL;
     }
 
-    /* save the global settings */
+    // save the global settings; for that we should hold only the VirtualBox lock
     alock.release();
+    AutoWriteLock vboxLock(m->pParent COMMA_LOCKVAL_SRC_POS);
     return rc = m->pParent->saveSettings();
 #else
     /* Note: The GUI depends on this method returning E_NOTIMPL with no
@@ -1761,7 +1763,9 @@ HRESULT Host::onUSBDeviceFilterChange(HostUSBDeviceFilter *aFilter,
         }
 
         // save the global settings... yeah, on every single filter property change
+        // for that we should hold only the VirtualBox lock
         alock.release();
+        AutoWriteLock vboxLock(m->pParent COMMA_LOCKVAL_SRC_POS);
         return m->pParent->saveSettings();
     }
 
