@@ -6,7 +6,7 @@
  */
 
 /*
- * Copyright (C) 2006-2008 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2008 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -15,27 +15,28 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 #include "HostNetworkInterfaceImpl.h"
+#include "AutoCaller.h"
 #include "Logging.h"
 #include "netif.h"
 
-#ifndef RT_OS_WINDOWS
-#include <arpa/inet.h>
-# ifdef RT_OS_FREEBSD
-#  include <netinet/in.h> /* INADDR_NONE */
-# endif
-#endif /* RT_OS_WINDOWS */
+#ifdef RT_OS_FREEBSD
+# include <netinet/in.h> /* INADDR_NONE */
+#endif /* RT_OS_FREEBSD */
 
 // constructor / destructor
 /////////////////////////////////////////////////////////////////////////////
 
-DEFINE_EMPTY_CTOR_DTOR (HostNetworkInterface)
+HostNetworkInterface::HostNetworkInterface()
+    : mVBox(NULL)
+{
+}
+
+HostNetworkInterface::~HostNetworkInterface()
+{
+}
 
 HRESULT HostNetworkInterface::FinalConstruct()
 {
@@ -62,8 +63,8 @@ HRESULT HostNetworkInterface::init (Bstr aInterfaceName, Guid aGuid, HostNetwork
     LogFlowThisFunc(("aInterfaceName={%ls}, aGuid={%s}\n",
                       aInterfaceName.raw(), aGuid.toString().raw()));
 
-    ComAssertRet (aInterfaceName, E_INVALIDARG);
-    ComAssertRet (!aGuid.isEmpty(), E_INVALIDARG);
+    ComAssertRet(aInterfaceName, E_INVALIDARG);
+    ComAssertRet(!aGuid.isEmpty(), E_INVALIDARG);
 
     /* Enclose the state transition NotReady->InInit->Ready */
     AutoInitSpan autoInitSpan(this);
@@ -86,7 +87,7 @@ HRESULT HostNetworkInterface::updateConfig ()
 {
     NETIFINFO info;
     int rc = NetIfGetConfig(this, &info);
-    if(RT_SUCCESS(rc))
+    if (RT_SUCCESS(rc))
     {
         m.realIPAddress = m.IPAddress = info.IPAddress.u;
         m.realNetworkMask = m.networkMask = info.IPNetMask.u;
@@ -119,9 +120,9 @@ HRESULT HostNetworkInterface::init (Bstr aInterfaceName, HostNetworkInterfaceTyp
 //    LogFlowThisFunc(("aInterfaceName={%ls}, aGuid={%s}\n",
 //                      aInterfaceName.raw(), aGuid.toString().raw()));
 
-//    ComAssertRet (aInterfaceName, E_INVALIDARG);
-//    ComAssertRet (!aGuid.isEmpty(), E_INVALIDARG);
-    ComAssertRet (pIf, E_INVALIDARG);
+//    ComAssertRet(aInterfaceName, E_INVALIDARG);
+//    ComAssertRet(!aGuid.isEmpty(), E_INVALIDARG);
+    ComAssertRet(pIf, E_INVALIDARG);
 
     /* Enclose the state transition NotReady->InInit->Ready */
     AutoInitSpan autoInitSpan(this);
@@ -166,7 +167,7 @@ STDMETHODIMP HostNetworkInterface::COMGETTER(Name) (BSTR *aInterfaceName)
     CheckComArgOutPointerValid(aInterfaceName);
 
     AutoCaller autoCaller(this);
-    CheckComRCReturnRC(autoCaller.rc());
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
 
     mInterfaceName.cloneTo(aInterfaceName);
 
@@ -184,7 +185,7 @@ STDMETHODIMP HostNetworkInterface::COMGETTER(Id) (BSTR *aGuid)
     CheckComArgOutPointerValid(aGuid);
 
     AutoCaller autoCaller(this);
-    CheckComRCReturnRC(autoCaller.rc());
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
 
     mGuid.toUtf16().cloneTo(aGuid);
 
@@ -196,7 +197,7 @@ STDMETHODIMP HostNetworkInterface::COMGETTER(DhcpEnabled) (BOOL *aDhcpEnabled)
     CheckComArgOutPointerValid(aDhcpEnabled);
 
     AutoCaller autoCaller(this);
-    CheckComRCReturnRC(autoCaller.rc());
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
 
     *aDhcpEnabled = m.dhcpEnabled;
 
@@ -215,11 +216,11 @@ STDMETHODIMP HostNetworkInterface::COMGETTER(IPAddress) (BSTR *aIPAddress)
     CheckComArgOutPointerValid(aIPAddress);
 
     AutoCaller autoCaller(this);
-    CheckComRCReturnRC(autoCaller.rc());
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
 
     if (m.IPAddress == 0)
     {
-        Bstr(VBOXNET_IPV4ADDR_DEFAULT).detachTo(aIPAddress);
+        getDefaultIPv4Address(mInterfaceName).detachTo(aIPAddress);
         return S_OK;
     }
 
@@ -230,7 +231,7 @@ STDMETHODIMP HostNetworkInterface::COMGETTER(IPAddress) (BSTR *aIPAddress)
     tmp.s_addr = m.IPAddress;
 #endif
     char *addr = inet_ntoa(tmp);
-    if(addr)
+    if (addr)
     {
         Bstr(addr).detachTo(aIPAddress);
         return S_OK;
@@ -250,7 +251,7 @@ STDMETHODIMP HostNetworkInterface::COMGETTER(NetworkMask) (BSTR *aNetworkMask)
     CheckComArgOutPointerValid(aNetworkMask);
 
     AutoCaller autoCaller(this);
-    CheckComRCReturnRC(autoCaller.rc());
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
 
     if (m.networkMask == 0)
     {
@@ -265,7 +266,7 @@ STDMETHODIMP HostNetworkInterface::COMGETTER(NetworkMask) (BSTR *aNetworkMask)
     tmp.s_addr = m.networkMask;
 #endif
     char *addr = inet_ntoa(tmp);
-    if(addr)
+    if (addr)
     {
         Bstr(addr).detachTo(aNetworkMask);
         return S_OK;
@@ -297,7 +298,7 @@ STDMETHODIMP HostNetworkInterface::COMGETTER(IPV6Address) (BSTR *aIPV6Address)
     CheckComArgOutPointerValid(aIPV6Address);
 
     AutoCaller autoCaller(this);
-    CheckComRCReturnRC(autoCaller.rc());
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
 
     m.IPV6Address.cloneTo(aIPV6Address);
 
@@ -315,7 +316,7 @@ STDMETHODIMP HostNetworkInterface::COMGETTER(IPV6NetworkMaskPrefixLength) (ULONG
     CheckComArgOutPointerValid(aIPV6NetworkMaskPrefixLength);
 
     AutoCaller autoCaller(this);
-    CheckComRCReturnRC(autoCaller.rc());
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
 
     *aIPV6NetworkMaskPrefixLength = m.IPV6NetworkMaskPrefixLength;
 
@@ -333,7 +334,7 @@ STDMETHODIMP HostNetworkInterface::COMGETTER(HardwareAddress) (BSTR *aHardwareAd
     CheckComArgOutPointerValid(aHardwareAddress);
 
     AutoCaller autoCaller(this);
-    CheckComRCReturnRC(autoCaller.rc());
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
 
     m.hardwareAddress.cloneTo(aHardwareAddress);
 
@@ -351,7 +352,7 @@ STDMETHODIMP HostNetworkInterface::COMGETTER(MediumType) (HostNetworkInterfaceMe
     CheckComArgOutPointerValid(aType);
 
     AutoCaller autoCaller(this);
-    CheckComRCReturnRC(autoCaller.rc());
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
 
     *aType = m.mediumType;
 
@@ -369,7 +370,7 @@ STDMETHODIMP HostNetworkInterface::COMGETTER(Status) (HostNetworkInterfaceStatus
     CheckComArgOutPointerValid(aStatus);
 
     AutoCaller autoCaller(this);
-    CheckComRCReturnRC(autoCaller.rc());
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
 
     *aStatus = m.status;
 
@@ -387,7 +388,7 @@ STDMETHODIMP HostNetworkInterface::COMGETTER(InterfaceType) (HostNetworkInterfac
     CheckComArgOutPointerValid(aType);
 
     AutoCaller autoCaller(this);
-    CheckComRCReturnRC(autoCaller.rc());
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
 
     *aType = mIfType;
 
@@ -398,7 +399,7 @@ STDMETHODIMP HostNetworkInterface::COMGETTER(InterfaceType) (HostNetworkInterfac
 STDMETHODIMP HostNetworkInterface::COMGETTER(NetworkName) (BSTR *aNetworkName)
 {
     AutoCaller autoCaller(this);
-    CheckComRCReturnRC(autoCaller.rc());
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
 
     Utf8Str utf8Name("HostInterfaceNetworking-");
     utf8Name.append(Utf8Str(mInterfaceName)) ;
@@ -414,7 +415,7 @@ STDMETHODIMP HostNetworkInterface::EnableStaticIpConfig (IN_BSTR aIPAddress, IN_
     return E_NOTIMPL;
 #else
     AutoCaller autoCaller(this);
-    CheckComRCReturnRC(autoCaller.rc());
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
 
     if (Bstr(aIPAddress).isEmpty())
     {
@@ -424,9 +425,9 @@ STDMETHODIMP HostNetworkInterface::EnableStaticIpConfig (IN_BSTR aIPAddress, IN_
             if (RT_SUCCESS(rc))
             {
                 m.realIPAddress = 0;
-                if (FAILED(mVBox->SetExtraData(Bstr(Utf8StrFmt("HostOnly/%ls/IPAddress", mInterfaceName.raw())), Bstr(""))))
+                if (FAILED(mVBox->SetExtraData(BstrFmt("HostOnly/%ls/IPAddress", mInterfaceName.raw()), NULL)))
                     return E_FAIL;
-                if (FAILED(mVBox->SetExtraData(Bstr(Utf8StrFmt("HostOnly/%ls/IPNetMask", mInterfaceName.raw())), Bstr(""))))
+                if (FAILED(mVBox->SetExtraData(BstrFmt("HostOnly/%ls/IPNetMask", mInterfaceName.raw()), NULL)))
                     return E_FAIL;
                 return S_OK;
             }
@@ -437,13 +438,13 @@ STDMETHODIMP HostNetworkInterface::EnableStaticIpConfig (IN_BSTR aIPAddress, IN_
 
     ULONG ip, mask;
     ip = inet_addr(Utf8Str(aIPAddress).raw());
-    if(ip != INADDR_NONE)
+    if (ip != INADDR_NONE)
     {
         if (Bstr(aNetMask).isEmpty())
             mask = 0xFFFFFF;
         else
             mask = inet_addr(Utf8Str(aNetMask).raw());
-        if(mask != INADDR_NONE)
+        if (mask != INADDR_NONE)
         {
             if (m.realIPAddress == ip && m.realNetworkMask == mask)
                 return S_OK;
@@ -460,7 +461,7 @@ STDMETHODIMP HostNetworkInterface::EnableStaticIpConfig (IN_BSTR aIPAddress, IN_
             }
             else
             {
-                LogRel(("Failed to EnableStaticIpConfig with rc=%Vrc\n", rc));
+                LogRel(("Failed to EnableStaticIpConfig with rc=%Rrc\n", rc));
                 return rc == VERR_NOT_IMPLEMENTED ? E_NOTIMPL : E_FAIL;
             }
 
@@ -481,7 +482,7 @@ STDMETHODIMP HostNetworkInterface::EnableStaticIpConfigV6 (IN_BSTR aIPV6Address,
         return E_INVALIDARG;
 
     AutoCaller autoCaller(this);
-    CheckComRCReturnRC(autoCaller.rc());
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
 
     int rc = S_OK;
     if (m.realIPV6Address != aIPV6Address || m.realIPV6PrefixLength != aIPV6MaskPrefixLength)
@@ -491,7 +492,7 @@ STDMETHODIMP HostNetworkInterface::EnableStaticIpConfigV6 (IN_BSTR aIPV6Address,
         rc = NetIfEnableStaticIpConfigV6(mVBox, this, m.IPV6Address, aIPV6Address, aIPV6MaskPrefixLength);
         if (RT_FAILURE(rc))
         {
-            LogRel(("Failed to EnableStaticIpConfigV6 with rc=%Vrc\n", rc));
+            LogRel(("Failed to EnableStaticIpConfigV6 with rc=%Rrc\n", rc));
             return rc == VERR_NOT_IMPLEMENTED ? E_NOTIMPL : E_FAIL;
         }
         else
@@ -516,12 +517,12 @@ STDMETHODIMP HostNetworkInterface::EnableDynamicIpConfig ()
     return E_NOTIMPL;
 #else
     AutoCaller autoCaller(this);
-    CheckComRCReturnRC(autoCaller.rc());
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
 
     int rc = NetIfEnableDynamicIpConfig(mVBox, this);
     if (RT_FAILURE(rc))
     {
-        LogRel(("Failed to EnableDynamicIpConfig with rc=%Vrc\n", rc));
+        LogRel(("Failed to EnableDynamicIpConfig with rc=%Rrc\n", rc));
         return rc == VERR_NOT_IMPLEMENTED ? E_NOTIMPL : E_FAIL;
     }
     return S_OK;
@@ -534,12 +535,12 @@ STDMETHODIMP HostNetworkInterface::DhcpRediscover ()
     return E_NOTIMPL;
 #else
     AutoCaller autoCaller(this);
-    CheckComRCReturnRC(autoCaller.rc());
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
 
     int rc = NetIfDhcpRediscover(mVBox, this);
     if (RT_FAILURE(rc))
     {
-        LogRel(("Failed to DhcpRediscover with rc=%Vrc\n", rc));
+        LogRel(("Failed to DhcpRediscover with rc=%Rrc\n", rc));
         return rc == VERR_NOT_IMPLEMENTED ? E_NOTIMPL : E_FAIL;
     }
     return S_OK;
@@ -550,8 +551,8 @@ HRESULT HostNetworkInterface::setVirtualBox(VirtualBox *pVBox)
 {
     HRESULT hrc;
     AutoCaller autoCaller(this);
-    CheckComRCReturnRC(autoCaller.rc());
-    mVBox = pVBox;
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
+    unconst(mVBox) = pVBox;
 
     /* If IPv4 address hasn't been initialized */
     if (m.IPAddress == 0)
@@ -560,7 +561,7 @@ HRESULT HostNetworkInterface::setVirtualBox(VirtualBox *pVBox)
         hrc = mVBox->GetExtraData(Bstr(Utf8StrFmt("HostOnly/%ls/IPAddress", mInterfaceName.raw())), tmpAddr.asOutParam());
         hrc = mVBox->GetExtraData(Bstr(Utf8StrFmt("HostOnly/%ls/IPNetMask", mInterfaceName.raw())), tmpMask.asOutParam());
         if (tmpAddr.isEmpty())
-            tmpAddr = Bstr(VBOXNET_IPV4ADDR_DEFAULT);
+            tmpAddr = getDefaultIPv4Address(mInterfaceName);
         if (tmpMask.isEmpty())
             tmpMask = Bstr(VBOXNET_IPV4MASK_DEFAULT);
         m.IPAddress = inet_addr(Utf8Str(tmpAddr).raw());
