@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2010 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -13,27 +13,45 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 #include "ParallelPortImpl.h"
 #include "MachineImpl.h"
 #include "VirtualBoxImpl.h"
-#include "Logging.h"
 
 #include <iprt/string.h>
-#include <iprt/cpputils.h>
+#include <iprt/cpp/utils.h>
 
 #include <VBox/settings.h>
 
+#include "AutoStateDep.h"
+#include "AutoCaller.h"
+#include "Logging.h"
+
+////////////////////////////////////////////////////////////////////////////////
+//
+// ParallelPort private data definition
+//
+////////////////////////////////////////////////////////////////////////////////
+
+struct ParallelPort::Data
+{
+    Data()
+        : fModified(false),
+          pMachine(NULL)
+    { }
+
+    bool                                    fModified;
+
+    Machine * const                         pMachine;
+    const ComObjPtr<ParallelPort>           pPeer;
+
+    Backupable<settings::ParallelPort>      bd;
+};
+
 // constructor / destructor
 /////////////////////////////////////////////////////////////////////////////
 
-DEFINE_EMPTY_CTOR_DTOR (ParallelPort)
-
 HRESULT ParallelPort::FinalConstruct()
 {
     return S_OK;
@@ -52,23 +70,25 @@ void ParallelPort::FinalRelease()
  *
  *  @param aParent  Handle of the parent object.
  */
-HRESULT ParallelPort::init (Machine *aParent, ULONG aSlot)
+HRESULT ParallelPort::init(Machine *aParent, ULONG aSlot)
 {
     LogFlowThisFunc(("aParent=%p, aSlot=%d\n", aParent, aSlot));
 
-    ComAssertRet (aParent, E_INVALIDARG);
+    ComAssertRet(aParent, E_INVALIDARG);
 
     /* Enclose the state transition NotReady->InInit->Ready */
     AutoInitSpan autoInitSpan(this);
     AssertReturn(autoInitSpan.isOk(), E_FAIL);
 
-    unconst(mParent) = aParent;
-    /* mPeer is left null */
+    m = new Data;
 
-    mData.allocate();
+    unconst(m->pMachine) = aParent;
+    /* m->pPeer is left null */
+
+    m->bd.allocate();
 
     /* initialize data */
-    mData->mSlot = aSlot;
+    m->bd->ulSlot = aSlot;
 
     /* Confirm a successful initialization */
     autoInitSpan.setSucceeded();
@@ -86,24 +106,26 @@ HRESULT ParallelPort::init (Machine *aParent, ULONG aSlot)
  *
  *  @note Locks @a aThat object for reading.
  */
-HRESULT ParallelPort::init (Machine *aParent, ParallelPort *aThat)
+HRESULT ParallelPort::init(Machine *aParent, ParallelPort *aThat)
 {
     LogFlowThisFunc(("aParent=%p, aThat=%p\n", aParent, aThat));
 
-    ComAssertRet (aParent && aThat, E_INVALIDARG);
+    ComAssertRet(aParent && aThat, E_INVALIDARG);
 
     /* Enclose the state transition NotReady->InInit->Ready */
     AutoInitSpan autoInitSpan(this);
     AssertReturn(autoInitSpan.isOk(), E_FAIL);
 
-    unconst(mParent) = aParent;
-    unconst(mPeer) = aThat;
+    m = new Data;
+
+    unconst(m->pMachine) = aParent;
+    unconst(m->pPeer) = aThat;
 
     AutoCaller thatCaller (aThat);
     AssertComRCReturnRC(thatCaller.rc());
 
-    AutoReadLock thatLock (aThat);
-    mData.share (aThat->mData);
+    AutoReadLock thatLock(aThat COMMA_LOCKVAL_SRC_POS);
+    m->bd.share(aThat->m->bd);
 
     /* Confirm a successful initialization */
     autoInitSpan.setSucceeded();
@@ -118,24 +140,26 @@ HRESULT ParallelPort::init (Machine *aParent, ParallelPort *aThat)
  *
  *  @note Locks @a aThat object for reading.
  */
-HRESULT ParallelPort::initCopy (Machine *aParent, ParallelPort *aThat)
+HRESULT ParallelPort::initCopy(Machine *aParent, ParallelPort *aThat)
 {
     LogFlowThisFunc(("aParent=%p, aThat=%p\n", aParent, aThat));
 
-    ComAssertRet (aParent && aThat, E_INVALIDARG);
+    ComAssertRet(aParent && aThat, E_INVALIDARG);
 
     /* Enclose the state transition NotReady->InInit->Ready */
     AutoInitSpan autoInitSpan(this);
     AssertReturn(autoInitSpan.isOk(), E_FAIL);
 
-    unconst(mParent) = aParent;
-    /* mPeer is left null */
+    m = new Data;
 
-    AutoCaller thatCaller (aThat);
+    unconst(m->pMachine) = aParent;
+    /* m->pPeer is left null */
+
+    AutoCaller thatCaller(aThat);
     AssertComRCReturnRC(thatCaller.rc());
 
-    AutoReadLock thatLock (aThat);
-    mData.attachCopy (aThat->mData);
+    AutoReadLock thatLock(aThat COMMA_LOCKVAL_SRC_POS);
+    m->bd.attachCopy(aThat->m->bd);
 
     /* Confirm a successful initialization */
     autoInitSpan.setSucceeded();
@@ -156,10 +180,232 @@ void ParallelPort::uninit()
     if (autoUninitSpan.uninitDone())
         return;
 
-    mData.free();
+    m->bd.free();
 
-    unconst(mPeer).setNull();
-    unconst(mParent).setNull();
+    unconst(m->pPeer) = NULL;
+    unconst(m->pMachine) = NULL;
+
+    delete m;
+    m = NULL;
+}
+
+// IParallelPort properties
+/////////////////////////////////////////////////////////////////////////////
+
+STDMETHODIMP ParallelPort::COMGETTER(Enabled) (BOOL *aEnabled)
+{
+    CheckComArgOutPointerValid(aEnabled);
+
+    AutoCaller autoCaller(this);
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
+
+    AutoReadLock alock(this COMMA_LOCKVAL_SRC_POS);
+
+    *aEnabled = m->bd->fEnabled;
+
+    return S_OK;
+}
+
+STDMETHODIMP ParallelPort::COMSETTER(Enabled) (BOOL aEnabled)
+{
+    LogFlowThisFunc(("aEnabled=%RTbool\n", aEnabled));
+
+    AutoCaller autoCaller(this);
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
+
+    /* the machine needs to be mutable */
+    AutoMutableStateDependency adep(m->pMachine);
+    if (FAILED(adep.rc())) return adep.rc();
+
+    AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
+
+    if (m->bd->fEnabled != !!aEnabled)
+    {
+        if (aEnabled &&
+            m->bd->strPath.isEmpty())
+            return setError(E_INVALIDARG,
+                            tr("Cannot enable the parallel port %d because the port path is empty or null"),
+                            m->bd->ulSlot);
+
+        m->bd.backup();
+        m->bd->fEnabled = !!aEnabled;
+
+        m->fModified = true;
+        // leave the lock before informing callbacks
+        alock.release();
+
+        AutoWriteLock mlock(m->pMachine COMMA_LOCKVAL_SRC_POS);
+        m->pMachine->setModified(Machine::IsModified_ParallelPorts);
+        mlock.release();
+
+        m->pMachine->onParallelPortChange(this);
+    }
+
+    return S_OK;
+}
+
+STDMETHODIMP ParallelPort::COMGETTER(Slot) (ULONG *aSlot)
+{
+    CheckComArgOutPointerValid(aSlot);
+
+    AutoCaller autoCaller(this);
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
+
+    AutoReadLock alock(this COMMA_LOCKVAL_SRC_POS);
+
+    *aSlot = m->bd->ulSlot;
+
+    return S_OK;
+}
+
+STDMETHODIMP ParallelPort::COMGETTER(IRQ) (ULONG *aIRQ)
+{
+    CheckComArgOutPointerValid(aIRQ);
+
+    AutoCaller autoCaller(this);
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
+
+    AutoReadLock alock(this COMMA_LOCKVAL_SRC_POS);
+
+    *aIRQ = m->bd->ulIRQ;
+
+    return S_OK;
+}
+
+STDMETHODIMP ParallelPort::COMSETTER(IRQ)(ULONG aIRQ)
+{
+    /* check IRQ limits
+     * (when changing this, make sure it corresponds to XML schema */
+    if (aIRQ > 255)
+        return setError(E_INVALIDARG,
+                        tr("Invalid IRQ number of the parallel port %d: %lu (must be in range [0, %lu])"),
+                        m->bd->ulSlot, aIRQ, 255);
+
+    AutoCaller autoCaller(this);
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
+
+    /* the machine needs to be mutable */
+    AutoMutableStateDependency adep(m->pMachine);
+    if (FAILED(adep.rc())) return adep.rc();
+
+    AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
+
+    if (m->bd->ulIRQ != aIRQ)
+    {
+        m->bd.backup();
+        m->bd->ulIRQ = aIRQ;
+
+        m->fModified = true;
+        // leave the lock before informing callbacks
+        alock.release();
+
+        AutoWriteLock mlock(m->pMachine COMMA_LOCKVAL_SRC_POS);
+        m->pMachine->setModified(Machine::IsModified_ParallelPorts);
+        mlock.release();
+
+        m->pMachine->onParallelPortChange(this);
+    }
+
+    return S_OK;
+}
+
+STDMETHODIMP ParallelPort::COMGETTER(IOBase) (ULONG *aIOBase)
+{
+    CheckComArgOutPointerValid(aIOBase);
+
+    AutoCaller autoCaller(this);
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
+
+    AutoReadLock alock(this COMMA_LOCKVAL_SRC_POS);
+
+    *aIOBase = m->bd->ulIOBase;
+
+    return S_OK;
+}
+
+STDMETHODIMP ParallelPort::COMSETTER(IOBase)(ULONG aIOBase)
+{
+    /* check IOBase limits
+     * (when changing this, make sure it corresponds to XML schema */
+    if (aIOBase > 0xFFFF)
+        return setError(E_INVALIDARG,
+                        tr("Invalid I/O port base address of the parallel port %d: %lu (must be in range [0, 0x%X])"),
+                        m->bd->ulSlot, aIOBase, 0, 0xFFFF);
+
+    AutoCaller autoCaller(this);
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
+
+    /* the machine needs to be mutable */
+    AutoMutableStateDependency adep(m->pMachine);
+    if (FAILED(adep.rc())) return adep.rc();
+
+    AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
+
+    if (m->bd->ulIOBase != aIOBase)
+    {
+        m->bd.backup();
+        m->bd->ulIOBase = aIOBase;
+
+        m->fModified = true;
+        // leave the lock before informing callbacks
+        alock.release();
+
+        AutoWriteLock mlock(m->pMachine COMMA_LOCKVAL_SRC_POS);
+        m->pMachine->setModified(Machine::IsModified_ParallelPorts);
+        mlock.release();
+
+        m->pMachine->onParallelPortChange(this);
+    }
+
+    return S_OK;
+}
+
+STDMETHODIMP ParallelPort::COMGETTER(Path) (BSTR *aPath)
+{
+    CheckComArgOutPointerValid(aPath);
+
+    AutoCaller autoCaller(this);
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
+
+    AutoReadLock alock(this COMMA_LOCKVAL_SRC_POS);
+
+    m->bd->strPath.cloneTo(aPath);
+
+    return S_OK;
+}
+
+STDMETHODIMP ParallelPort::COMSETTER(Path) (IN_BSTR aPath)
+{
+    AutoCaller autoCaller(this);
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
+
+    /* the machine needs to be mutable */
+    AutoMutableStateDependency adep(m->pMachine);
+    if (FAILED(adep.rc())) return adep.rc();
+
+    AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
+
+    Utf8Str str(aPath);
+    if (str != m->bd->strPath)
+    {
+        HRESULT rc = checkSetPath(str);
+        if (FAILED(rc)) return rc;
+
+        m->bd.backup();
+        m->bd->strPath = str;
+
+        m->fModified = true;
+        // leave the lock before informing callbacks
+        alock.release();
+
+        AutoWriteLock mlock(m->pMachine COMMA_LOCKVAL_SRC_POS);
+        m->pMachine->setModified(Machine::IsModified_ParallelPorts);
+        mlock.release();
+
+        return m->pMachine->onParallelPortChange(this);
+    }
+
+    return S_OK;
 }
 
 // public methods only for internal purposes
@@ -178,30 +424,10 @@ HRESULT ParallelPort::loadSettings(const settings::ParallelPort &data)
     AutoCaller autoCaller(this);
     AssertComRCReturnRC(autoCaller.rc());
 
-    AutoWriteLock alock(this);
+    AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
 
-    /* Note: we assume that the default values for attributes of optional
-     * nodes are assigned in the Data::Data() constructor and don't do it
-     * here. It implies that this method may only be called after constructing
-     * a new BIOSSettings object while all its data fields are in the default
-     * values. Exceptions are fields whose creation time defaults don't match
-     * values that should be applied when these fields are not explicitly set
-     * in the settings file (for backwards compatibility reasons). This takes
-     * place when a setting of a newly created object must default to A while
-     * the same setting of an object loaded from the old settings file must
-     * default to B. */
-
-    /* enabled (required) */
-    mData->mEnabled = data.fEnabled;
-    /* I/O base (required) */
-    mData->mIOBase = data.ulIOBase;
-    /* IRQ (required) */
-    mData->mIRQ = data.ulIRQ;
-    /* device path (optional, defaults to null) */
-    Bstr path(data.strPath);
-    HRESULT rc = checkSetPath (path);
-    CheckComRCReturnRC(rc);
-    mData->mPath = path;
+    // simply copy
+    *m->bd.data() = data;
 
     return S_OK;
 }
@@ -220,38 +446,36 @@ HRESULT ParallelPort::saveSettings(settings::ParallelPort &data)
     AutoCaller autoCaller(this);
     AssertComRCReturnRC(autoCaller.rc());
 
-    AutoReadLock alock(this);
+    AutoReadLock alock(this COMMA_LOCKVAL_SRC_POS);
 
-    data.fEnabled = !!mData->mEnabled;
-    data.ulIOBase = mData->mIOBase;
-    data.ulIRQ = mData->mIRQ;
-    data.strPath = mData->mPath;
+    // simply copy
+    data = *m->bd.data();
 
     return S_OK;
 }
 
+/**
+ * Returns true if any setter method has modified settings of this instance.
+ * @return
+ */
+bool ParallelPort::isModified()
+{
+    AutoReadLock alock(this COMMA_LOCKVAL_SRC_POS);
+    return m->fModified;
+}
+
 /**
  *  @note Locks this object for writing.
  */
-bool ParallelPort::rollback()
+void ParallelPort::rollback()
 {
     /* sanity */
     AutoCaller autoCaller(this);
-    AssertComRCReturn (autoCaller.rc(), false);
+    AssertComRCReturnVoid(autoCaller.rc());
 
-    AutoWriteLock alock(this);
+    AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
 
-    bool changed = false;
-
-    if (mData.isBackedUp())
-    {
-        /* we need to check all data to see whether anything will be changed
-         * after rollback */
-        changed = mData.hasActualChanges();
-        mData.rollback();
-    }
-
-    return changed;
+    m->bd.rollback();
 }
 
 /**
@@ -265,20 +489,20 @@ void ParallelPort::commit()
     AssertComRCReturnVoid (autoCaller.rc());
 
     /* sanity too */
-    AutoCaller peerCaller (mPeer);
+    AutoCaller peerCaller (m->pPeer);
     AssertComRCReturnVoid (peerCaller.rc());
 
-    /* lock both for writing since we modify both (mPeer is "master" so locked
+    /* lock both for writing since we modify both (m->pPeer is "master" so locked
      * first) */
-    AutoMultiWriteLock2 alock (mPeer, this);
+    AutoMultiWriteLock2 alock(m->pPeer, this COMMA_LOCKVAL_SRC_POS);
 
-    if (mData.isBackedUp())
+    if (m->bd.isBackedUp())
     {
-        mData.commit();
-        if (mPeer)
+        m->bd.commit();
+        if (m->pPeer)
         {
             /* attach new data to the peer and reshare it */
-            mPeer->mData.attach (mData);
+            m->pPeer->m->bd.attach(m->bd);
         }
     }
 }
@@ -287,7 +511,7 @@ void ParallelPort::commit()
  *  @note Locks this object for writing, together with the peer object
  *  represented by @a aThat (locked for reading).
  */
-void ParallelPort::copyFrom (ParallelPort *aThat)
+void ParallelPort::copyFrom(ParallelPort *aThat)
 {
     AssertReturnVoid (aThat != NULL);
 
@@ -301,245 +525,30 @@ void ParallelPort::copyFrom (ParallelPort *aThat)
 
     /* peer is not modified, lock it for reading (aThat is "master" so locked
      * first) */
-    AutoMultiLock2 alock (aThat->rlock(), this->wlock());
+    AutoReadLock rl(aThat COMMA_LOCKVAL_SRC_POS);
+    AutoWriteLock wl(this COMMA_LOCKVAL_SRC_POS);
 
     /* this will back up current data */
-    mData.assignCopy (aThat->mData);
-}
-
-// IParallelPort properties
-/////////////////////////////////////////////////////////////////////////////
-
-STDMETHODIMP ParallelPort::COMGETTER(Enabled) (BOOL *aEnabled)
-{
-    CheckComArgOutPointerValid(aEnabled);
-
-    AutoCaller autoCaller(this);
-    CheckComRCReturnRC(autoCaller.rc());
-
-    AutoReadLock alock(this);
-
-    *aEnabled = mData->mEnabled;
-
-    return S_OK;
-}
-
-STDMETHODIMP ParallelPort::COMSETTER(Enabled) (BOOL aEnabled)
-{
-    LogFlowThisFunc(("aEnabled=%RTbool\n", aEnabled));
-
-    AutoCaller autoCaller(this);
-    CheckComRCReturnRC(autoCaller.rc());
-
-    /* the machine needs to be mutable */
-    Machine::AutoMutableStateDependency adep (mParent);
-    CheckComRCReturnRC(adep.rc());
-
-    AutoWriteLock alock(this);
-
-    if (mData->mEnabled != aEnabled)
-    {
-        if (aEnabled &&
-            mData->mPath.isEmpty())
-            return setError (E_INVALIDARG,
-                        tr ("Cannot enable the parallel port %d "
-                            "because the port path is empty or null"),
-                        mData->mSlot);
-
-        mData.backup();
-        mData->mEnabled = aEnabled;
-
-        /* leave the lock before informing callbacks */
-        alock.unlock();
-
-        mParent->onParallelPortChange (this);
-    }
-
-    return S_OK;
-}
-
-STDMETHODIMP ParallelPort::COMGETTER(Slot) (ULONG *aSlot)
-{
-    CheckComArgOutPointerValid(aSlot);
-
-    AutoCaller autoCaller(this);
-    CheckComRCReturnRC(autoCaller.rc());
-
-    AutoReadLock alock(this);
-
-    *aSlot = mData->mSlot;
-
-    return S_OK;
-}
-
-STDMETHODIMP ParallelPort::COMGETTER(IRQ) (ULONG *aIRQ)
-{
-    CheckComArgOutPointerValid(aIRQ);
-
-    AutoCaller autoCaller(this);
-    CheckComRCReturnRC(autoCaller.rc());
-
-    AutoReadLock alock(this);
-
-    *aIRQ = mData->mIRQ;
-
-    return S_OK;
-}
-
-STDMETHODIMP ParallelPort::COMSETTER(IRQ)(ULONG aIRQ)
-{
-    /* check IRQ limits
-     * (when changing this, make sure it corresponds to XML schema */
-    if (aIRQ > 255)
-        return setError (E_INVALIDARG,
-            tr ("Invalid IRQ number of the parallel port %d: "
-                "%lu (must be in range [0, %lu])"),
-            mData->mSlot, aIRQ, 255);
-
-    AutoCaller autoCaller(this);
-    CheckComRCReturnRC(autoCaller.rc());
-
-    /* the machine needs to be mutable */
-    Machine::AutoMutableStateDependency adep (mParent);
-    CheckComRCReturnRC(adep.rc());
-
-    AutoWriteLock alock(this);
-
-    HRESULT rc = S_OK;
-    bool emitChangeEvent = false;
-
-    if (mData->mIRQ != aIRQ)
-    {
-        mData.backup();
-        mData->mIRQ = aIRQ;
-        emitChangeEvent = true;
-    }
-
-    if (emitChangeEvent)
-    {
-        /* leave the lock before informing callbacks */
-        alock.unlock();
-
-        mParent->onParallelPortChange (this);
-    }
-
-    return rc;
-}
-
-STDMETHODIMP ParallelPort::COMGETTER(IOBase) (ULONG *aIOBase)
-{
-    CheckComArgOutPointerValid(aIOBase);
-
-    AutoCaller autoCaller(this);
-    CheckComRCReturnRC(autoCaller.rc());
-
-    AutoReadLock alock(this);
-
-    *aIOBase = mData->mIOBase;
-
-    return S_OK;
-}
-
-STDMETHODIMP ParallelPort::COMSETTER(IOBase)(ULONG aIOBase)
-{
-    /* check IOBase limits
-     * (when changing this, make sure it corresponds to XML schema */
-    if (aIOBase > 0xFFFF)
-        return setError (E_INVALIDARG,
-            tr ("Invalid I/O port base address of the parallel port %d: "
-                "%lu (must be in range [0, 0x%X])"),
-            mData->mSlot, aIOBase, 0, 0xFFFF);
-
-    AutoCaller autoCaller(this);
-    CheckComRCReturnRC(autoCaller.rc());
-
-    /* the machine needs to be mutable */
-    Machine::AutoMutableStateDependency adep (mParent);
-    CheckComRCReturnRC(adep.rc());
-
-    AutoWriteLock alock(this);
-
-    HRESULT rc = S_OK;
-    bool emitChangeEvent = false;
-
-    if (mData->mIOBase != aIOBase)
-    {
-        mData.backup();
-        mData->mIOBase = aIOBase;
-        emitChangeEvent = true;
-    }
-
-    if (emitChangeEvent)
-    {
-        /* leave the lock before informing callbacks */
-        alock.unlock();
-
-        mParent->onParallelPortChange (this);
-    }
-
-    return rc;
-}
-
-STDMETHODIMP ParallelPort::COMGETTER(Path) (BSTR *aPath)
-{
-    CheckComArgOutPointerValid(aPath);
-
-    AutoCaller autoCaller(this);
-    CheckComRCReturnRC(autoCaller.rc());
-
-    AutoReadLock alock(this);
-
-    mData->mPath.cloneTo(aPath);
-
-    return S_OK;
+    m->bd.assignCopy(aThat->m->bd);
 }
 
 /**
  *  Validates COMSETTER(Path) arguments.
  */
-HRESULT ParallelPort::checkSetPath (CBSTR aPath)
+HRESULT ParallelPort::checkSetPath(const Utf8Str &str)
 {
     AssertReturn(isWriteLockOnCurrentThread(), E_FAIL);
 
-    if (mData->mEnabled &&
-        (aPath == NULL || *aPath == '\0'))
-        return setError (E_INVALIDARG,
-            tr ("Path of the parallel port %d may not be empty or null "
-                "when the port is enabled"),
-            mData->mSlot);
+    if (    m->bd->fEnabled
+         && str.isEmpty()
+       )
+        return setError(E_INVALIDARG,
+                        tr("Path of the parallel port %d may not be empty or null "
+                           "when the port is enabled"),
+                        m->bd->ulSlot);
 
     return S_OK;
 }
 
-STDMETHODIMP ParallelPort::COMSETTER(Path) (IN_BSTR aPath)
-{
-    AutoCaller autoCaller(this);
-    CheckComRCReturnRC(autoCaller.rc());
 
-    /* the machine needs to be mutable */
-    Machine::AutoMutableStateDependency adep (mParent);
-    CheckComRCReturnRC(adep.rc());
-
-    AutoWriteLock alock(this);
-
-    /* we treat empty as null when e.g. saving to XML, do the same here */
-    if (aPath && *aPath == '\0')
-        aPath = NULL;
-
-    if (mData->mPath != aPath)
-    {
-        HRESULT rc = checkSetPath (aPath);
-        CheckComRCReturnRC(rc);
-
-        mData.backup();
-        mData->mPath = aPath;
-
-        /* leave the lock before informing callbacks */
-        alock.unlock();
-
-        return mParent->onParallelPortChange (this);
-    }
-
-    return S_OK;
-}
 /* vi: set tabstop=4 shiftwidth=4 expandtab: */
