@@ -612,6 +612,10 @@ private:
     bool mVMPoweredOff : 1;
     /** true when vmstateChangeCallback shouldn't initiate a power down.  */
     bool mVMIsAlreadyPoweringOff : 1;
+    /** true if we already showed the snapshot folder size warning. */
+    bool mfSnapshotFolderSizeWarningShown : 1;
+    /** true if we already showed the snapshot folder ext4/xfs bug warning. */
+    bool mfSnapshotFolderExt4WarningShown : 1;
 
     /** The current network attachment type in the VM.
      * This doesn't have to match the network attachment type
