@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -22,28 +22,26 @@
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 /*******************************************************************************
 *   Header Files                                                               *
 *******************************************************************************/
-#include <VBox/sup.h>
+#ifndef RT_NO_GIP
+# include <VBox/sup.h>
+#endif
 #include <iprt/asm.h>
 #include <iprt/assert.h>
 #include <iprt/buildconfig.h>
 #include <iprt/system.h>
 
-# include <libxml/xmlmodule.h>
-# include <libxml/globals.h>
-# include <openssl/md5.h>
-# include <openssl/rc4.h>
-# include <openssl/pem.h>
-# include <openssl/x509.h>
-# include <openssl/rsa.h>
+#include <libxml/xmlmodule.h>
+#include <libxml/globals.h>
+#include <openssl/md5.h>
+#include <openssl/rc4.h>
+#include <openssl/pem.h>
+#include <openssl/x509.h>
+#include <openssl/rsa.h>
 
 
 /*******************************************************************************
@@ -51,9 +49,11 @@
 *******************************************************************************/
 PFNRT g_VBoxRTDeps[] =
 {
+#ifndef RT_NO_GIP
     (PFNRT)SUPR3Init,
     (PFNRT)SUPR3PageAllocEx,
     (PFNRT)SUPSemEventCreate,
+#endif
     (PFNRT)xmlModuleOpen,
     (PFNRT)MD5_Init,
     (PFNRT)RC4,
