@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2007 Sun Microsystems, Inc.
+ * Copyright (C) 2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -22,10 +22,6 @@
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 
@@ -119,7 +115,7 @@ RTDECL(int) RTOnce(PRTONCE pOnce, PFNRTONCE pfnOnce, void *pvUser1, void *pvUser
                  * the init code, if it isn't valid just do the yield/sleep thing.
                  */
                 RTSEMEVENTMULTI hEventMulti;
-                ASMAtomicUoReadSize(&pOnce->hEventMulti, &hEventMulti);
+                ASMAtomicUoReadHandle(&pOnce->hEventMulti, &hEventMulti);
                 if (hEventMulti != NIL_RTSEMEVENTMULTI)
                 {
                     fYieldSleep = false;
