@@ -1,10 +1,10 @@
-/* $Id: ldrNative-posix.cpp 29978 2008-04-21 17:24:28Z umoeller $ */
+/* $Id: ldrNative-posix.cpp $ */
 /** @file
  * IPRT - Binary Image Loader, POSIX native.
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -22,10 +22,6 @@
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 /*******************************************************************************
@@ -79,7 +75,7 @@ int rtldrNativeLoad(const char *pszFilename, uintptr_t *phHandle)
         *phHandle = (uintptr_t)pvMod;
         return VINF_SUCCESS;
     }
-    Log(("rtldrNativeLoad: dlopen('%s', RTLD_NOW | RTLD_LOCAL) failed: %s\n", pszFilename, dlerror()));
+    LogRel(("rtldrNativeLoad: dlopen('%s', RTLD_NOW | RTLD_LOCAL) failed: %s\n", pszFilename, dlerror()));
     return VERR_FILE_NOT_FOUND;
 }
 
