@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2008-2009 Sun Microsystems, Inc.
+ * Copyright (C) 2008-2009 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -22,10 +22,6 @@
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 
@@ -70,20 +66,40 @@ int main()
         return rc;
     RTTestBanner(hTest);
 
+    CHECKNETWORK("10.0.0/45",      VERR_INVALID_PARAMETER,          0,          0);
+    CHECKNETWORK("10.0.0/-45",     VERR_INVALID_PARAMETER,          0,          0);
     CHECKNETWORK("10.0.0/24",                VINF_SUCCESS, 0x0A000000, 0xFFFFFF00);
+    CHECKNETWORK("10..0.0/24",     VERR_INVALID_PARAMETER,          0,          0);
+    CHECKNETWORK(".10.0.0/24",     VERR_INVALID_PARAMETER,          0,          0);
+    CHECKNETWORK("10.0.0//24",     VERR_INVALID_PARAMETER,          0,          0);
     CHECKNETWORK("10.0.0/8",                 VINF_SUCCESS, 0x0A000000, 0xFF000000);
     CHECKNETWORK("10.0.0./24",     VERR_INVALID_PARAMETER,          0,          0);
     CHECKNETWORK("0.1.0/24",       VERR_INVALID_PARAMETER,          0,          0);
-    CHECKNETWORK("10.255.0.0/24",  VERR_INVALID_PARAMETER,          0,          0);
+    /* RFC 4632 s3.1: n.n.n.0/24, where n is an 8-bit decimal octet value*/
+    CHECKNETWORK("10.255.0.0/24",            VINF_SUCCESS, 0x0AFF0000, 0xFFFFFF00);
     CHECKNETWORK("10.1234.0.0/24", VERR_INVALID_PARAMETER,          0,          0);
     CHECKNETWORK("10.256.0.0/24",  VERR_INVALID_PARAMETER,          0,          0);
     CHECKNETWORK("10.0.0/3",       VERR_INVALID_PARAMETER,          0,          0);
-    CHECKNETWORK("10.1.2.3/8",               VINF_SUCCESS, 0x0A010203, 0xFF000000);
+    /* RFC 4632 s3.1: legacy "Class A" is n.0.0.0/8 */
+    CHECKNETWORK("10.1.2.3/8",     VERR_INVALID_PARAMETER,          0,          0);
+    CHECKNETWORK("10.1.2.4/30",              VINF_SUCCESS, 0x0A010204, 0xFFFFFFFC);
     CHECKNETWORK("10.0.0/29",      VERR_INVALID_PARAMETER,          0,          0);
     CHECKNETWORK("10.0.0/240",     VERR_INVALID_PARAMETER,          0,          0);
     CHECKNETWORK("10.0.0/24.",     VERR_INVALID_PARAMETER,          0,          0);
-    CHECKNETWORK("10.1.2/16",                VINF_SUCCESS, 0x0A010200, 0xFFFF0000);
+    /* RFC 4632 s3.1: legacy "Class B" is n.n.0.0/16 */
+    CHECKNETWORK("10.1.2/16",      VERR_INVALID_PARAMETER,          0,          0);
+    CHECKNETWORK("10.1/16",                  VINF_SUCCESS, 0x0A010000, 0xFFFF0000);
+    CHECKNETWORK("10.1.0.0/16",              VINF_SUCCESS, 0x0A010000, 0xFFFF0000);
+    CHECKNETWORK("10.1.0.0.0/16",  VERR_INVALID_PARAMETER,          0,          0);
     CHECKNETWORK("1.2.3.4",                  VINF_SUCCESS, 0x01020304, 0xFFFFFFFF);
+    CHECKNETWORK("1.2.3.255",                VINF_SUCCESS, 0x010203FF, 0xFFFFFFFF);
+    CHECKNETWORK("1.2.3.256",      VERR_INVALID_PARAMETER,          0,          0);
+    CHECKNETWORK("10.1.255/24",              VINF_SUCCESS, 0x0A01FF00, 0xFFFFFF00);
+    CHECKNETWORK("10.1.254/24",              VINF_SUCCESS, 0x0A01FE00, 0xFFFFFF00);
+    CHECKNETWORK("10.255.1/24",              VINF_SUCCESS, 0x0AFF0100, 0xFFFFFF00);
+    CHECKNETWORK("10.255.1.1/24",  VERR_INVALID_PARAMETER,          0,          0);
+    CHECKNETWORK("1.2",            VERR_INVALID_PARAMETER,          0,          0);
+    CHECKNETWORK("1.2.3.4.5",      VERR_INVALID_PARAMETER,          0,          0);
 
     return RTTestSummaryAndDestroy(hTest);
 }
