@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -13,10 +13,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 #ifndef ___tstHelp_h
@@ -40,7 +36,7 @@ RT_C_DECLS_END
     do { \
         if (off != RT_OFFSETOF(type, m)) \
         { \
-            printf("%#010x %s  Off by %d!! (off=%#x)\n", RT_OFFSETOF(type, m), #type "." #m, off - RT_OFFSETOF(type, m), off); \
+            printf("error! %#010x %s  Off by %d!! (off=%#x)\n", RT_OFFSETOF(type, m), #type "." #m, off - RT_OFFSETOF(type, m), off); \
             rc++; \
         } \
         /*else */ \
@@ -56,11 +52,11 @@ RT_C_DECLS_END
     do { \
         if (size != sizeof(type)) \
         { \
-            printf("sizeof(%s): %#x (%d)  Off by %d!!\n", #type, (int)sizeof(type), (int)sizeof(type), (int)(sizeof(type) - size)); \
+            printf("error! sizeof(%s): %#x (%d)  Off by %d!!\n", #type, (int)sizeof(type), (int)sizeof(type), (int)(sizeof(type) - size)); \
             rc++; \
         } \
         else \
-            printf("sizeof(%s): %#x (%d)\n", #type, (int)sizeof(type), (int)sizeof(type)); \
+            printf("info: sizeof(%s): %#x (%d)\n", #type, (int)sizeof(type), (int)sizeof(type)); \
     } while (0)
 
 /**
@@ -71,7 +67,7 @@ RT_C_DECLS_END
     { \
         if (RT_OFFSETOF(strct, member) & ((align) - 1) ) \
         { \
-            printf("%s::%s offset=%#x (%u) expected alignment %x, meaning %#x (%u) off\n", \
+            printf("error! %s::%s offset=%#x (%u) expected alignment %x, meaning %#x (%u) off\n", \
                    #strct, #member, \
                    (unsigned)RT_OFFSETOF(strct, member), \
                    (unsigned)RT_OFFSETOF(strct, member), \
@@ -89,7 +85,7 @@ RT_C_DECLS_END
     do { \
         if (RT_ALIGN_Z(sizeof(type), (align)) != sizeof(type)) \
         { \
-            printf("%s size=%#x (%u), align=%#x %#x (%u) bytes off\n", \
+            printf("error! %s size=%#x (%u), align=%#x %#x (%u) bytes off\n", \
                    #type, \
                    (unsigned)sizeof(type), \
                    (unsigned)sizeof(type), \
@@ -109,13 +105,13 @@ RT_C_DECLS_END
         strct *p; \
         if (sizeof(p->member.s) > sizeof(p->member.padding)) \
         { \
-            printf("padding of %s::%s is too small, padding=%d struct=%d correct=%d\n", #strct, #member, \
+            printf("error! padding of %s::%s is too small, padding=%d struct=%d correct=%d\n", #strct, #member, \
                    (int)sizeof(p->member.padding), (int)sizeof(p->member.s), (int)RT_ALIGN_Z(sizeof(p->member.s), (align))); \
             rc++; \
         } \
         else if (RT_ALIGN_Z(sizeof(p->member.padding), (align)) != sizeof(p->member.padding)) \
         { \
-            printf("padding of %s::%s is misaligned, padding=%d correct=%d\n", #strct, #member, \
+            printf("error! padding of %s::%s is misaligned, padding=%d correct=%d\n", #strct, #member, \
                    (int)sizeof(p->member.padding), (int)RT_ALIGN_Z(sizeof(p->member.s), (align))); \
             rc++; \
         } \
@@ -130,7 +126,7 @@ RT_C_DECLS_END
         strct *p; \
         if (sizeof(p->s) > sizeof(p->padding)) \
         { \
-            printf("padding of %s is too small, padding=%d struct=%d correct=%d\n", #strct, \
+            printf("error! padding of %s is too small, padding=%d struct=%d correct=%d\n", #strct, \
                    (int)sizeof(p->padding), (int)sizeof(p->s), (int)RT_ALIGN_Z(sizeof(p->s), 64)); \
             rc++; \
         } \
@@ -145,7 +141,7 @@ RT_C_DECLS_END
         strct *p; \
         if (sizeof(p->member) > sizeof(p->pad_member)) \
         { \
-            printf("padding of %s::%s is too small, padding=%d struct=%d\n", #strct, #member, \
+            printf("error! padding of %s::%s is too small, padding=%d struct=%d\n", #strct, #member, \
                    (int)sizeof(p->pad_member), (int)sizeof(p->member)); \
             rc++; \
         } \
